# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v22.enums.types import customer_status


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "CustomerClient",
    },
)


class CustomerClient(proto.Message):
    r"""A link between the given customer and a client customer.
    CustomerClients only exist for manager customers. All direct and
    indirect client customers are included, as well as the manager
    itself.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Output only. The resource name of the customer client.
            CustomerClient resource names have the form:
            ``customers/{customer_id}/customerClients/{client_customer_id}``
        client_customer (str):
            Output only. The resource name of the
            client-customer which is linked to the given
            customer. Read only.

            This field is a member of `oneof`_ ``_client_customer``.
        hidden (bool):
            Output only. Specifies whether this is a `hidden
            account <https://support.google.com/google-ads/answer/7519830>`__.
            Read only.

            This field is a member of `oneof`_ ``_hidden``.
        level (int):
            Output only. Distance between given customer
            and client. For self link, the level value will
            be 0. Read only.

            This field is a member of `oneof`_ ``_level``.
        time_zone (str):
            Output only. Common Locale Data Repository (CLDR) string
            representation of the time zone of the client, for example,
            America/Los_Angeles. Read only.

            This field is a member of `oneof`_ ``_time_zone``.
        test_account (bool):
            Output only. Identifies if the client is a
            test account. Read only.

            This field is a member of `oneof`_ ``_test_account``.
        manager (bool):
            Output only. Identifies if the client is a
            manager. Read only.

            This field is a member of `oneof`_ ``_manager``.
        descriptive_name (str):
            Output only. Descriptive name for the client.
            Read only.

            This field is a member of `oneof`_ ``_descriptive_name``.
        currency_code (str):
            Output only. Currency code (for example,
            'USD', 'EUR') for the client. Read only.

            This field is a member of `oneof`_ ``_currency_code``.
        id (int):
            Output only. The ID of the client customer.
            Read only.

            This field is a member of `oneof`_ ``_id``.
        applied_labels (MutableSequence[str]):
            Output only. The resource names of the labels owned by the
            requesting customer that are applied to the client customer.
            Label resource names have the form:

            ``customers/{customer_id}/labels/{label_id}``
        status (google.ads.googleads.v22.enums.types.CustomerStatusEnum.CustomerStatus):
            Output only. The status of the client
            customer. Read only.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    client_customer: str = proto.Field(
        proto.STRING,
        number=12,
        optional=True,
    )
    hidden: bool = proto.Field(
        proto.BOOL,
        number=13,
        optional=True,
    )
    level: int = proto.Field(
        proto.INT64,
        number=14,
        optional=True,
    )
    time_zone: str = proto.Field(
        proto.STRING,
        number=15,
        optional=True,
    )
    test_account: bool = proto.Field(
        proto.BOOL,
        number=16,
        optional=True,
    )
    manager: bool = proto.Field(
        proto.BOOL,
        number=17,
        optional=True,
    )
    descriptive_name: str = proto.Field(
        proto.STRING,
        number=18,
        optional=True,
    )
    currency_code: str = proto.Field(
        proto.STRING,
        number=19,
        optional=True,
    )
    id: int = proto.Field(
        proto.INT64,
        number=20,
        optional=True,
    )
    applied_labels: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=21,
    )
    status: customer_status.CustomerStatusEnum.CustomerStatus = proto.Field(
        proto.ENUM,
        number=22,
        enum=customer_status.CustomerStatusEnum.CustomerStatus,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.enums",
    marshal="google.ads.googleads.v22",
    manifest={
        "AssetGroupPrimaryStatusReasonEnum",
    },
)


class AssetGroupPrimaryStatusReasonEnum(proto.Message):
    r"""Container for enum describing possible asset group primary
    status reasons.

    """

    class AssetGroupPrimaryStatusReason(proto.Enum):
        r"""Enum describing the possible asset group primary status
        reasons. Provides reasons into why an asset group is not serving
        or not serving optimally. It will be empty when the asset group
        is serving without issues.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            ASSET_GROUP_PAUSED (2):
                The user-specified asset group status is
                paused. Contributes to
                AssetGroupPrimaryStatus.PAUSED
            ASSET_GROUP_REMOVED (3):
                The user-specified asset group status is
                removed. Contributes to
                AssetGroupPrimaryStatus.REMOVED.
            CAMPAIGN_REMOVED (4):
                The user-specified campaign status is removed. Contributes
                to AssetGroupPrimaryStatus.NOT_ELIGIBLE.
            CAMPAIGN_PAUSED (5):
                The user-specified campaign status is paused. Contributes to
                AssetGroupPrimaryStatus.NOT_ELIGIBLE.
            CAMPAIGN_PENDING (6):
                The user-specified time for this campaign to start is in the
                future. Contributes to AssetGroupPrimaryStatus.NOT_ELIGIBLE.
            CAMPAIGN_ENDED (7):
                The user-specified time for this campaign to end has passed.
                Contributes to AssetGroupPrimaryStatus.NOT_ELIGIBLE.
            ASSET_GROUP_LIMITED (8):
                The asset group is approved but only serves
                in limited capacity due to policies. Contributes
                to AssetGroupPrimaryStatus.LIMITED.
            ASSET_GROUP_DISAPPROVED (9):
                The asset group has been marked as disapproved. Contributes
                to AssetGroupPrimaryStatus.NOT_ELIGIBLE.
            ASSET_GROUP_UNDER_REVIEW (10):
                The asset group has not completed policy
                review. Contributes to
                AssetGroupPrimaryStatus.PENDING.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        ASSET_GROUP_PAUSED = 2
        ASSET_GROUP_REMOVED = 3
        CAMPAIGN_REMOVED = 4
        CAMPAIGN_PAUSED = 5
        CAMPAIGN_PENDING = 6
        CAMPAIGN_ENDED = 7
        ASSET_GROUP_LIMITED = 8
        ASSET_GROUP_DISAPPROVED = 9
        ASSET_GROUP_UNDER_REVIEW = 10


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This example imports conversion adjustments for existing conversions.

To set up a conversion action, run the add_conversion_action.py example.
"""


import argparse
import sys
from typing import Optional, Iterable

from google.protobuf.any_pb2 import Any

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.enums.types.conversion_adjustment_type import (
    ConversionAdjustmentTypeEnum,
)
from google.ads.googleads.v22.errors.types.errors import (
    GoogleAdsError,
    GoogleAdsFailure,
)
from google.ads.googleads.v22.services.services.conversion_action_service import (
    ConversionActionServiceClient,
)
from google.ads.googleads.v22.services.services.conversion_adjustment_upload_service import (
    ConversionAdjustmentUploadServiceClient,
)
from google.ads.googleads.v22.services.types.conversion_adjustment_upload_service import (
    ConversionAdjustment,
    ConversionAdjustmentResult,
    UploadConversionAdjustmentsRequest,
    UploadConversionAdjustmentsResponse,
)


# [START upload_conversion_adjustment]
def main(
    client: GoogleAdsClient,
    customer_id: str,
    conversion_action_id: str,
    adjustment_type: str,
    order_id: str,
    adjustment_date_time: str,
    restatement_value: Optional[str] = None,
) -> None:
    """The main method that creates all necessary entities for the example.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        conversion_action_id: the ID of the conversion action to upload the
            adjustment to.
        adjustment_type: the adjustment type, e.g. " "RETRACTION, RESTATEMENT.
        order_id: the transaction ID of the conversion to adjust. Strongly
            recommended instead of using gclid and conversion_date_time.
        adjustment_date_time: the date and time of the adjustment.
        restatement_value: the adjusted value for adjustment type RESTATEMENT.
    """
    conversion_adjustment_type_enum: ConversionAdjustmentTypeEnum = (
        client.enums.ConversionAdjustmentTypeEnum
    )
    # Determine the adjustment type.
    conversion_adjustment_type: int = conversion_adjustment_type_enum[
        adjustment_type
    ].value

    # Applies the conversion adjustment to the existing conversion.
    conversion_adjustment: ConversionAdjustment = client.get_type(
        "ConversionAdjustment"
    )
    conversion_action_service: ConversionActionServiceClient = (
        client.get_service("ConversionActionService")
    )
    conversion_adjustment.conversion_action = (
        conversion_action_service.conversion_action_path(
            customer_id, conversion_action_id
        )
    )
    conversion_adjustment.adjustment_type = conversion_adjustment_type
    conversion_adjustment.adjustment_date_time = adjustment_date_time

    # Sets the order_id to identify the conversion to adjust.
    conversion_adjustment.order_id = order_id

    # As an alternative to setting order_id, you can provide a
    # gclid_date_time_pair, but setting order_id instead is strongly recommended.
    # conversion_adjustment.gclid_date_time_pair.gclid = gclid
    # conversion_adjustment.gclid_date_time_pair.conversion_date_time = (
    #     conversion_date_time
    # )

    # Sets adjusted value for adjustment type RESTATEMENT.
    if (
        restatement_value
        and conversion_adjustment_type
        == conversion_adjustment_type_enum.RESTATEMENT.value
    ):
        conversion_adjustment.restatement_value.adjusted_value = float(
            restatement_value
        )

    # Uploads the click conversion. Partial failure should always be set to
    # true.
    service: ConversionAdjustmentUploadServiceClient = client.get_service(
        "ConversionAdjustmentUploadService"
    )
    request: UploadConversionAdjustmentsRequest = client.get_type(
        "UploadConversionAdjustmentsRequest"
    )
    request.customer_id = customer_id
    request.conversion_adjustments.append(conversion_adjustment)
    # Enables partial failure (must be true)
    request.partial_failure = True

    response: UploadConversionAdjustmentsResponse = (
        service.upload_conversion_adjustments(request=request)
    )

    # Extracts the partial failure error if present on the response.
    error_details = None
    if response.partial_failure_error:
        error_details: Iterable[Any] = response.partial_failure_error.details

    i: int
    conversion_adjustment_result: ConversionAdjustmentResult
    for i, conversion_adjustment_result in enumerate(response.results):
        # If there's a GoogleAdsFailure in error_details at this position then
        # the uploaded operation failed and we print the error message.
        if error_details and error_details[i]:
            error_detail: Any = error_details[i]
            failure_message: GoogleAdsFailure = client.get_type(
                "GoogleAdsFailure"
            )
            # Parse the string into a GoogleAdsFailure message instance.
            # To access class-only methods on the message we retrieve its type.
            google_ads_failure_class: GoogleAdsFailure = type(failure_message)
            failure_object: GoogleAdsFailure = (
                google_ads_failure_class.deserialize(error_detail.value)
            )

            error: GoogleAdsError
            for error in failure_object.errors:
                # Construct and print a string that details which element in
                # the operation list failed (by index number) as well as the
                # error message and error code.
                print(
                    "A partial failure at index "
                    f"{error.location.field_path_elements[0].index} occurred "
                    f"\nError message: {error.message}\nError code: "
                    f"{error.error_code}"
                )
        else:
            print(
                "Uploaded conversion adjustment for conversion action "
                f"'{conversion_adjustment_result.conversion_action}' and order "
                f"ID '{conversion_adjustment_result.order_id}'."
            )
            # [END upload_conversion_adjustment]


if __name__ == "__main__":
    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    parser: argparse.ArgumentParser = argparse.ArgumentParser(
        description="Uploads a conversion adjustment."
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-a",
        "--conversion_action_id",
        type=str,
        required=True,
        help="The ID of the conversion action to upload the adjustment to.",
    )
    parser.add_argument(
        "-d",
        "--adjustment_type",
        type=str,
        required=True,
        choices=[
            e.name
            for e in googleads_client.enums.ConversionAdjustmentTypeEnum
            if e.name not in ("UNSPECIFIED", "UNKNOWN")
        ],
        help="The adjustment type, e.g. " "RETRACTION, RESTATEMENT",
    )
    parser.add_argument(
        "-o",
        "--order_id",
        type=str,
        required=True,
        help=(
            "The transaction ID of the conversion to adjust. Required if the "
            "conversion being adjusted meets the criteria described at: "
            "https://developers.google.com/google-ads/api/docs/conversions/upload-adjustments#requirements."
        ),
    )
    parser.add_argument(
        "-v",
        "--adjustment_date_time",
        type=str,
        required=True,
        help=(
            "The date and time of the adjustment. The format is "
            "'yyyy-mm-dd hh:mm:ss+|-hh:mm', e.g. '2019-01-01 12:32:45-08:00'"
        ),
    )
    # Optional: Specify an adjusted value for adjustment type RESTATEMENT.
    # This value will be ignored if you specify RETRACTION as adjustment type.
    parser.add_argument(
        "-r",
        "--restatement_value",
        type=str,
        required=False,
        help="The adjusted value for adjustment type RESTATEMENT.",
    )
    args: argparse.Namespace = parser.parse_args()

    try:
        main(
            googleads_client,
            args.customer_id,
            args.conversion_action_id,
            args.adjustment_type,
            args.order_id,
            args.adjustment_date_time,
            args.restatement_value,
        )
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.enums",
    marshal="google.ads.googleads.v22",
    manifest={
        "PromotionExtensionOccasionEnum",
    },
)


class PromotionExtensionOccasionEnum(proto.Message):
    r"""Container for enum describing a promotion extension occasion.
    For more information about the occasions  check:

    https://support.google.com/google-ads/answer/7367521

    """

    class PromotionExtensionOccasion(proto.Enum):
        r"""A promotion extension occasion.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            NEW_YEARS (2):
                New Year's.
            CHINESE_NEW_YEAR (3):
                Chinese New Year.
            VALENTINES_DAY (4):
                Valentine's Day.
            EASTER (5):
                Easter.
            MOTHERS_DAY (6):
                Mother's Day.
            FATHERS_DAY (7):
                Father's Day.
            LABOR_DAY (8):
                Labor Day.
            BACK_TO_SCHOOL (9):
                Back To School.
            HALLOWEEN (10):
                Halloween.
            BLACK_FRIDAY (11):
                Black Friday.
            CYBER_MONDAY (12):
                Cyber Monday.
            CHRISTMAS (13):
                Christmas.
            BOXING_DAY (14):
                Boxing Day.
            INDEPENDENCE_DAY (15):
                Independence Day in any country.
            NATIONAL_DAY (16):
                National Day in any country.
            END_OF_SEASON (17):
                End of any season.
            WINTER_SALE (18):
                Winter Sale.
            SUMMER_SALE (19):
                Summer sale.
            FALL_SALE (20):
                Fall Sale.
            SPRING_SALE (21):
                Spring Sale.
            RAMADAN (22):
                Ramadan.
            EID_AL_FITR (23):
                Eid al-Fitr.
            EID_AL_ADHA (24):
                Eid al-Adha.
            SINGLES_DAY (25):
                Singles Day.
            WOMENS_DAY (26):
                Women's Day.
            HOLI (27):
                Holi.
            PARENTS_DAY (28):
                Parent's Day.
            ST_NICHOLAS_DAY (29):
                St. Nicholas Day.
            CARNIVAL (30):
                Carnival.
            EPIPHANY (31):
                Epiphany, also known as Three Kings' Day.
            ROSH_HASHANAH (32):
                Rosh Hashanah.
            PASSOVER (33):
                Passover.
            HANUKKAH (34):
                Hanukkah.
            DIWALI (35):
                Diwali.
            NAVRATRI (36):
                Navratri.
            SONGKRAN (37):
                Available in Thai: Songkran.
            YEAR_END_GIFT (38):
                Available in Japanese: Year-end Gift.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        NEW_YEARS = 2
        CHINESE_NEW_YEAR = 3
        VALENTINES_DAY = 4
        EASTER = 5
        MOTHERS_DAY = 6
        FATHERS_DAY = 7
        LABOR_DAY = 8
        BACK_TO_SCHOOL = 9
        HALLOWEEN = 10
        BLACK_FRIDAY = 11
        CYBER_MONDAY = 12
        CHRISTMAS = 13
        BOXING_DAY = 14
        INDEPENDENCE_DAY = 15
        NATIONAL_DAY = 16
        END_OF_SEASON = 17
        WINTER_SALE = 18
        SUMMER_SALE = 19
        FALL_SALE = 20
        SPRING_SALE = 21
        RAMADAN = 22
        EID_AL_FITR = 23
        EID_AL_ADHA = 24
        SINGLES_DAY = 25
        WOMENS_DAY = 26
        HOLI = 27
        PARENTS_DAY = 28
        ST_NICHOLAS_DAY = 29
        CARNIVAL = 30
        EPIPHANY = 31
        ROSH_HASHANAH = 32
        PASSOVER = 33
        HANUKKAH = 34
        DIWALI = 35
        NAVRATRI = 36
        SONGKRAN = 37
        YEAR_END_GIFT = 38


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v20.enums.types import (
    local_services_lead_credit_state,
)
from google.ads.googleads.v20.enums.types import local_services_lead_status
from google.ads.googleads.v20.enums.types import local_services_lead_type


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.resources",
    marshal="google.ads.googleads.v20",
    manifest={
        "LocalServicesLead",
        "ContactDetails",
        "Note",
        "CreditDetails",
    },
)


class LocalServicesLead(proto.Message):
    r"""Data from Local Services Lead.
    Contains details of Lead which is generated when user calls,
    messages or books service from advertiser.
    More info: https://ads.google.com/local-services-ads


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Output only. The resource name of the local services lead
            data. Local Services Lead resource name have the form

            ``customers/{customer_id}/localServicesLead/{local_services_lead_id}``
        id (int):
            Output only. ID of this Lead.
        category_id (str):
            Output only. Service category of the lead. For example:
            ``xcat:service_area_business_hvac``,
            ``xcat:service_area_business_real_estate_agent``, etc. For
            more details see:
            https://developers.google.com/google-ads/api/data/codes-formats#local_services_ids
        service_id (str):
            Output only. Service for the category. For example:
            ``buyer_agent``, ``seller_agent`` for the category of
            ``xcat:service_area_business_real_estate_agent``.
        contact_details (google.ads.googleads.v20.resources.types.ContactDetails):
            Output only. Lead's contact details.
        lead_type (google.ads.googleads.v20.enums.types.LocalServicesLeadTypeEnum.LeadType):
            Output only. Type of Local Services lead:
            phone, message, booking, etc.
        lead_status (google.ads.googleads.v20.enums.types.LocalServicesLeadStatusEnum.LeadStatus):
            Output only. Current status of lead.
        creation_date_time (str):
            Output only. The date time at which lead was
            created by Local Services Ads. The format is
            "YYYY-MM-DD HH:MM:SS" in the Google Ads
            account's timezone. Examples: "2018-03-05
            09:15:00" or "2018-02-01 14:34:30".
        locale (str):
            Output only. Language used by the Local
            Services provider linked to lead. See
            https://developers.google.com/google-ads/api/data/codes-formats#locales
        note (google.ads.googleads.v20.resources.types.Note):
            Output only. Note added by advertiser for the
            lead.

            This field is a member of `oneof`_ ``_note``.
        lead_charged (bool):
            Output only. True if the advertiser was
            charged for the lead.
        credit_details (google.ads.googleads.v20.resources.types.CreditDetails):
            Output only. Credit details of the lead.

            This field is a member of `oneof`_ ``_credit_details``.
        lead_feedback_submitted (bool):
            Output only. True if the advertiser submitted
            feedback for the lead.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=2,
    )
    category_id: str = proto.Field(
        proto.STRING,
        number=3,
    )
    service_id: str = proto.Field(
        proto.STRING,
        number=4,
    )
    contact_details: "ContactDetails" = proto.Field(
        proto.MESSAGE,
        number=5,
        message="ContactDetails",
    )
    lead_type: local_services_lead_type.LocalServicesLeadTypeEnum.LeadType = (
        proto.Field(
            proto.ENUM,
            number=6,
            enum=local_services_lead_type.LocalServicesLeadTypeEnum.LeadType,
        )
    )
    lead_status: (
        local_services_lead_status.LocalServicesLeadStatusEnum.LeadStatus
    ) = proto.Field(
        proto.ENUM,
        number=7,
        enum=local_services_lead_status.LocalServicesLeadStatusEnum.LeadStatus,
    )
    creation_date_time: str = proto.Field(
        proto.STRING,
        number=8,
    )
    locale: str = proto.Field(
        proto.STRING,
        number=9,
    )
    note: "Note" = proto.Field(
        proto.MESSAGE,
        number=10,
        optional=True,
        message="Note",
    )
    lead_charged: bool = proto.Field(
        proto.BOOL,
        number=11,
    )
    credit_details: "CreditDetails" = proto.Field(
        proto.MESSAGE,
        number=12,
        optional=True,
        message="CreditDetails",
    )
    lead_feedback_submitted: bool = proto.Field(
        proto.BOOL,
        number=13,
    )


class ContactDetails(proto.Message):
    r"""Fields containing consumer contact details.

    Attributes:
        phone_number (str):
            Output only. Consumer phone number in E164
            format.
        email (str):
            Output only. Consumer email address.
        consumer_name (str):
            Output only. Consumer name if consumer
            provided name from Message or Booking form on
            google.com
    """

    phone_number: str = proto.Field(
        proto.STRING,
        number=1,
    )
    email: str = proto.Field(
        proto.STRING,
        number=2,
    )
    consumer_name: str = proto.Field(
        proto.STRING,
        number=3,
    )


class Note(proto.Message):
    r"""Represents a note added to a lead by the advertiser.
    Advertisers can edit notes, which will reset edit time and
    change description.

    Attributes:
        edit_date_time (str):
            Output only. The date time when lead note was
            edited. The format is "YYYY-MM-DD HH:MM:SS" in
            the Google Ads account's timezone. Examples:

            "2018-03-05 09:15:00" or "2018-02-01 14:34:30".
        description (str):
            Output only. Content of lead note.
    """

    edit_date_time: str = proto.Field(
        proto.STRING,
        number=1,
    )
    description: str = proto.Field(
        proto.STRING,
        number=2,
    )


class CreditDetails(proto.Message):
    r"""Represents the credit details of a lead.

    Attributes:
        credit_state (google.ads.googleads.v20.enums.types.LocalServicesCreditStateEnum.CreditState):
            Output only. Credit state of the lead.
        credit_state_last_update_date_time (str):
            Output only. The date time when the credit
            state of the lead was last updated. The format
            is "YYYY-MM-DD HH:MM:SS" in the Google Ads
            account's timezone. Examples: "2018-03-05
            09:15:00" or "2018-02-01 14:34:30".
    """

    credit_state: (
        local_services_lead_credit_state.LocalServicesCreditStateEnum.CreditState
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=local_services_lead_credit_state.LocalServicesCreditStateEnum.CreditState,
    )
    credit_state_last_update_date_time: str = proto.Field(
        proto.STRING,
        number=2,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v22.resources.types import custom_audience
from google.protobuf import field_mask_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.services",
    marshal="google.ads.googleads.v22",
    manifest={
        "MutateCustomAudiencesRequest",
        "CustomAudienceOperation",
        "MutateCustomAudiencesResponse",
        "MutateCustomAudienceResult",
    },
)


class MutateCustomAudiencesRequest(proto.Message):
    r"""Request message for
    [CustomAudienceService.MutateCustomAudiences][google.ads.googleads.v22.services.CustomAudienceService.MutateCustomAudiences].

    Attributes:
        customer_id (str):
            Required. The ID of the customer whose custom
            audiences are being modified.
        operations (MutableSequence[google.ads.googleads.v22.services.types.CustomAudienceOperation]):
            Required. The list of operations to perform
            on individual custom audiences.
        validate_only (bool):
            If true, the request is validated but not
            executed. Only errors are returned, not results.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    operations: MutableSequence["CustomAudienceOperation"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="CustomAudienceOperation",
        )
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=3,
    )


class CustomAudienceOperation(proto.Message):
    r"""A single operation (create, update) on a custom audience.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            FieldMask that determines which resource
            fields are modified in an update.
        create (google.ads.googleads.v22.resources.types.CustomAudience):
            Create operation: No resource name is
            expected for the new custom audience.

            This field is a member of `oneof`_ ``operation``.
        update (google.ads.googleads.v22.resources.types.CustomAudience):
            Update operation: The custom audience is
            expected to have a valid resource name.

            This field is a member of `oneof`_ ``operation``.
        remove (str):
            Remove operation: A resource name for the removed custom
            audience is expected, in this format:

            ``customers/{customer_id}/customAudiences/{custom_audience_id}``

            This field is a member of `oneof`_ ``operation``.
    """

    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=4,
        message=field_mask_pb2.FieldMask,
    )
    create: custom_audience.CustomAudience = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="operation",
        message=custom_audience.CustomAudience,
    )
    update: custom_audience.CustomAudience = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="operation",
        message=custom_audience.CustomAudience,
    )
    remove: str = proto.Field(
        proto.STRING,
        number=3,
        oneof="operation",
    )


class MutateCustomAudiencesResponse(proto.Message):
    r"""Response message for custom audience mutate.

    Attributes:
        results (MutableSequence[google.ads.googleads.v22.services.types.MutateCustomAudienceResult]):
            All results for the mutate.
    """

    results: MutableSequence["MutateCustomAudienceResult"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message="MutateCustomAudienceResult",
        )
    )


class MutateCustomAudienceResult(proto.Message):
    r"""The result for the custom audience mutate.

    Attributes:
        resource_name (str):
            Returned for successful operations.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v19.common.types import asset_policy
from google.ads.googleads.v19.enums.types import (
    asset_performance_label as gage_asset_performance_label,
)
from google.ads.googleads.v19.enums.types import served_asset_field_type


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.common",
    marshal="google.ads.googleads.v19",
    manifest={
        "AdTextAsset",
        "AdImageAsset",
        "AdVideoAsset",
        "AdVideoAssetInfo",
        "AdVideoAssetInventoryPreferences",
        "AdMediaBundleAsset",
        "AdDemandGenCarouselCardAsset",
        "AdCallToActionAsset",
        "AdAppDeepLinkAsset",
    },
)


class AdTextAsset(proto.Message):
    r"""A text asset used inside an ad.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        text (str):
            Asset text.

            This field is a member of `oneof`_ ``_text``.
        pinned_field (google.ads.googleads.v19.enums.types.ServedAssetFieldTypeEnum.ServedAssetFieldType):
            The pinned field of the asset. This restricts
            the asset to only serve within this field.
            Multiple assets can be pinned to the same field.
            An asset that is unpinned or pinned to a
            different field will not serve in a field where
            some other asset has been pinned.
        asset_performance_label (google.ads.googleads.v19.enums.types.AssetPerformanceLabelEnum.AssetPerformanceLabel):
            The performance label of this text asset.
        policy_summary_info (google.ads.googleads.v19.common.types.AdAssetPolicySummary):
            The policy summary of this text asset.
    """

    text: str = proto.Field(
        proto.STRING,
        number=4,
        optional=True,
    )
    pinned_field: (
        served_asset_field_type.ServedAssetFieldTypeEnum.ServedAssetFieldType
    ) = proto.Field(
        proto.ENUM,
        number=2,
        enum=served_asset_field_type.ServedAssetFieldTypeEnum.ServedAssetFieldType,
    )
    asset_performance_label: (
        gage_asset_performance_label.AssetPerformanceLabelEnum.AssetPerformanceLabel
    ) = proto.Field(
        proto.ENUM,
        number=5,
        enum=gage_asset_performance_label.AssetPerformanceLabelEnum.AssetPerformanceLabel,
    )
    policy_summary_info: asset_policy.AdAssetPolicySummary = proto.Field(
        proto.MESSAGE,
        number=6,
        message=asset_policy.AdAssetPolicySummary,
    )


class AdImageAsset(proto.Message):
    r"""An image asset used inside an ad.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        asset (str):
            The Asset resource name of this image.

            This field is a member of `oneof`_ ``_asset``.
    """

    asset: str = proto.Field(
        proto.STRING,
        number=2,
        optional=True,
    )


class AdVideoAsset(proto.Message):
    r"""A video asset used inside an ad.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        asset (str):
            The Asset resource name of this video.

            This field is a member of `oneof`_ ``_asset``.
        ad_video_asset_info (google.ads.googleads.v19.common.types.AdVideoAssetInfo):
            Contains info fields for this AdVideoAsset.

            This field is a member of `oneof`_ ``_ad_video_asset_info``.
    """

    asset: str = proto.Field(
        proto.STRING,
        number=2,
        optional=True,
    )
    ad_video_asset_info: "AdVideoAssetInfo" = proto.Field(
        proto.MESSAGE,
        number=4,
        optional=True,
        message="AdVideoAssetInfo",
    )


class AdVideoAssetInfo(proto.Message):
    r"""Contains info fields for AdVideoAssets.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        ad_video_asset_inventory_preferences (google.ads.googleads.v19.common.types.AdVideoAssetInventoryPreferences):
            List of inventory preferences for this
            AdVideoAsset. This field can only be set for
            DiscoveryVideoResponsiveAd. The video assets
            with an inventory asset preference set will be
            preferred over other videos from the same ad
            during serving time. For example, consider this
            ad being served for a specific inventory. The
            server will first try to match an eligible video
            with a matching preference for that inventory.
            Videos with no preferences are chosen only when
            a video with matching preference and eligible
            for a given ad slot can be found.

            This field is a member of `oneof`_ ``_ad_video_asset_inventory_preferences``.
    """

    ad_video_asset_inventory_preferences: "AdVideoAssetInventoryPreferences" = (
        proto.Field(
            proto.MESSAGE,
            number=1,
            optional=True,
            message="AdVideoAssetInventoryPreferences",
        )
    )


class AdVideoAssetInventoryPreferences(proto.Message):
    r"""YouTube Video Asset inventory preferences.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        in_feed_preference (bool):
            When true, YouTube Video Asset with this
            inventory preference will be preferred when
            choosing a video to serve In Feed.

            This field is a member of `oneof`_ ``_in_feed_preference``.
        in_stream_preference (bool):
            When true, YouTube Video Asset with this
            inventory preference will be preferred when
            choosing a video to serve In Stream.

            This field is a member of `oneof`_ ``_in_stream_preference``.
        shorts_preference (bool):
            When true, YouTube Video Asset with this
            inventory preference will be preferred when
            choosing a video to serve on YouTube Shorts.

            This field is a member of `oneof`_ ``_shorts_preference``.
    """

    in_feed_preference: bool = proto.Field(
        proto.BOOL,
        number=1,
        optional=True,
    )
    in_stream_preference: bool = proto.Field(
        proto.BOOL,
        number=2,
        optional=True,
    )
    shorts_preference: bool = proto.Field(
        proto.BOOL,
        number=3,
        optional=True,
    )


class AdMediaBundleAsset(proto.Message):
    r"""A media bundle asset used inside an ad.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        asset (str):
            The Asset resource name of this media bundle.

            This field is a member of `oneof`_ ``_asset``.
    """

    asset: str = proto.Field(
        proto.STRING,
        number=2,
        optional=True,
    )


class AdDemandGenCarouselCardAsset(proto.Message):
    r"""A Demand Gen carousel card asset used inside an ad.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        asset (str):
            The Asset resource name of this discovery
            carousel card.

            This field is a member of `oneof`_ ``_asset``.
    """

    asset: str = proto.Field(
        proto.STRING,
        number=1,
        optional=True,
    )


class AdCallToActionAsset(proto.Message):
    r"""A call to action asset used inside an ad.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        asset (str):
            The Asset resource name of this call to
            action asset.

            This field is a member of `oneof`_ ``_asset``.
    """

    asset: str = proto.Field(
        proto.STRING,
        number=1,
        optional=True,
    )


class AdAppDeepLinkAsset(proto.Message):
    r"""An app deep link used inside an ad.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        asset (str):
            The Asset resource name of this app deep link
            asset.

            This field is a member of `oneof`_ ``_asset``.
    """

    asset: str = proto.Field(
        proto.STRING,
        number=1,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v22.enums.types import (
    asset_set_type as gage_asset_set_type,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "AssetSetTypeView",
    },
)


class AssetSetTypeView(proto.Message):
    r"""An asset set type view.
    This view reports non-overcounted metrics for each asset set
    type. Child asset set types are not included in this report.
    Their stats are aggregated under the parent asset set type.

    Attributes:
        resource_name (str):
            Output only. The resource name of the asset set type view.
            Asset set type view resource names have the form:

            ``customers/{customer_id}/assetSetTypeViews/{asset_set_type}``
        asset_set_type (google.ads.googleads.v22.enums.types.AssetSetTypeEnum.AssetSetType):
            Output only. The asset set type of the asset
            set type view.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    asset_set_type: gage_asset_set_type.AssetSetTypeEnum.AssetSetType = (
        proto.Field(
            proto.ENUM,
            number=3,
            enum=gage_asset_set_type.AssetSetTypeEnum.AssetSetType,
        )
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.common",
    marshal="google.ads.googleads.v20",
    manifest={
        "ClickLocation",
    },
)


class ClickLocation(proto.Message):
    r"""Location criteria associated with a click.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        city (str):
            The city location criterion associated with
            the impression.

            This field is a member of `oneof`_ ``_city``.
        country (str):
            The country location criterion associated
            with the impression.

            This field is a member of `oneof`_ ``_country``.
        metro (str):
            The metro location criterion associated with
            the impression.

            This field is a member of `oneof`_ ``_metro``.
        most_specific (str):
            The most specific location criterion
            associated with the impression.

            This field is a member of `oneof`_ ``_most_specific``.
        region (str):
            The region location criterion associated with
            the impression.

            This field is a member of `oneof`_ ``_region``.
    """

    city: str = proto.Field(
        proto.STRING,
        number=6,
        optional=True,
    )
    country: str = proto.Field(
        proto.STRING,
        number=7,
        optional=True,
    )
    metro: str = proto.Field(
        proto.STRING,
        number=8,
        optional=True,
    )
    most_specific: str = proto.Field(
        proto.STRING,
        number=9,
        optional=True,
    )
    region: str = proto.Field(
        proto.STRING,
        number=10,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This example adds a hotel callout extension asset to a specific account."""


import argparse
from typing import List
import sys


from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.services.types.asset_service import AssetOperation
from google.ads.googleads.v22.resources.types.asset import Asset
from google.ads.googleads.v22.services.types.customer_asset_service import (
    CustomerAssetOperation,
)
from google.ads.googleads.v22.resources.types.customer_asset import (
    CustomerAsset,
)


def main(client: GoogleAdsClient, customer_id: str, language_code: str) -> None:
    """The main method that creates all necessary entities for the example.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        language_code: the language of the hotel callout feed item text.
    """
    # Creates assets for the hotel callout assets.
    asset_resource_names: List[str] = add_hotel_callout_assets(
        client, customer_id, language_code
    )

    # Adds the assets at the account level, so these will serve in all eligible
    # campaigns.
    link_asset_to_account(client, customer_id, asset_resource_names)


def add_hotel_callout_assets(
    client: GoogleAdsClient, customer_id: str, language_code: str
) -> List[str]:
    """Creates new assets for the hotel callout.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        language_code: the language of the hotel callout feed item text.

    Returns:
        a list of asset resource names.
    """
    operations: List[AssetOperation] = []
    # Create a hotel callout asset operation for each of the below texts.
    for text in ["Activities", "Facilities"]:
        operation: AssetOperation = client.get_type("AssetOperation")
        asset: Asset = operation.create
        asset.hotel_callout_asset.text = text
        asset.hotel_callout_asset.language_code = language_code
        operations.append(operation)

    asset_service = client.get_service("AssetService")
    # Issues the create request to create the assets.
    response = asset_service.mutate_assets(
        customer_id=customer_id, operations=operations
    )
    resource_names: List[str] = [
        result.resource_name for result in response.results
    ]

    # Prints information about the result.
    for resource_name in resource_names:
        print(
            "Created hotel callout asset with resource name "
            f"'{resource_name}'."
        )

    return resource_names


def link_asset_to_account(
    client: GoogleAdsClient, customer_id: str, resource_names: List[str]
) -> None:
    """Links Asset at the Customer level to serve in all eligible campaigns.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        resource_names: a list of asset resource names.
    """
    # Creates a CustomerAsset link for each Asset resource name provided.
    operations: List[CustomerAssetOperation] = []
    for resource_name in resource_names:
        operation: CustomerAssetOperation = client.get_type(
            "CustomerAssetOperation"
        )
        asset: CustomerAsset = operation.create
        asset.asset = resource_name
        asset.field_type = client.enums.AssetFieldTypeEnum.HOTEL_CALLOUT
        operations.append(operation)

    customer_asset_service = client.get_service("CustomerAssetService")
    response = customer_asset_service.mutate_customer_assets(
        customer_id=customer_id, operations=operations
    )

    for result in response.results:
        print(
            "Added a customer asset with resource name "
            f"'{result.resource_name}'."
        )


if __name__ == "__main__":
    parser: argparse.ArgumentParser = argparse.ArgumentParser(
        description=(
            "Adds a hotel callout asset and adds the asset to the given "
            "account."
        )
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID",
    )
    parser.add_argument(
        "-l",
        "--language_code",
        type=str,
        required=True,
        help=(
            "The language of the text on the hotel callout feed item. For a "
            "list of supported languages see: "
            "https://developers.google.com/hotels/hotel-ads/api-reference/language-codes."
        ),
    )
    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(googleads_client, args.customer_id, args.language_code)
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v21.resources.types import (
    geo_target_constant as gagr_geo_target_constant,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.services",
    marshal="google.ads.googleads.v21",
    manifest={
        "SuggestGeoTargetConstantsRequest",
        "SuggestGeoTargetConstantsResponse",
        "GeoTargetConstantSuggestion",
    },
)


class SuggestGeoTargetConstantsRequest(proto.Message):
    r"""Request message for
    [GeoTargetConstantService.SuggestGeoTargetConstants][google.ads.googleads.v21.services.GeoTargetConstantService.SuggestGeoTargetConstants].

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        locale (str):
            If possible, returned geo targets are
            translated using this locale. If not, en is used
            by default. This is also used as a hint for
            returned geo targets.

            This field is a member of `oneof`_ ``_locale``.
        country_code (str):
            Returned geo targets are restricted to this
            country code.

            This field is a member of `oneof`_ ``_country_code``.
        location_names (google.ads.googleads.v21.services.types.SuggestGeoTargetConstantsRequest.LocationNames):
            The location names to search by. At most 25
            names can be set.

            This field is a member of `oneof`_ ``query``.
        geo_targets (google.ads.googleads.v21.services.types.SuggestGeoTargetConstantsRequest.GeoTargets):
            The geo target constant resource names to
            filter by.

            This field is a member of `oneof`_ ``query``.
    """

    class LocationNames(proto.Message):
        r"""A list of location names.

        Attributes:
            names (MutableSequence[str]):
                A list of location names.
        """

        names: MutableSequence[str] = proto.RepeatedField(
            proto.STRING,
            number=2,
        )

    class GeoTargets(proto.Message):
        r"""A list of geo target constant resource names.

        Attributes:
            geo_target_constants (MutableSequence[str]):
                A list of geo target constant resource names.
        """

        geo_target_constants: MutableSequence[str] = proto.RepeatedField(
            proto.STRING,
            number=2,
        )

    locale: str = proto.Field(
        proto.STRING,
        number=6,
        optional=True,
    )
    country_code: str = proto.Field(
        proto.STRING,
        number=7,
        optional=True,
    )
    location_names: LocationNames = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="query",
        message=LocationNames,
    )
    geo_targets: GeoTargets = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="query",
        message=GeoTargets,
    )


class SuggestGeoTargetConstantsResponse(proto.Message):
    r"""Response message for
    [GeoTargetConstantService.SuggestGeoTargetConstants][google.ads.googleads.v21.services.GeoTargetConstantService.SuggestGeoTargetConstants].

    Attributes:
        geo_target_constant_suggestions (MutableSequence[google.ads.googleads.v21.services.types.GeoTargetConstantSuggestion]):
            Geo target constant suggestions.
    """

    geo_target_constant_suggestions: MutableSequence[
        "GeoTargetConstantSuggestion"
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="GeoTargetConstantSuggestion",
    )


class GeoTargetConstantSuggestion(proto.Message):
    r"""A geo target constant suggestion.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        locale (str):
            The language this GeoTargetConstantSuggestion
            is currently translated to. It affects the name
            of geo target fields. For example, if locale=en,
            then name=Spain. If locale=es, then name=España.
            The default locale will be returned if no
            translation exists for the locale in the
            request.

            This field is a member of `oneof`_ ``_locale``.
        reach (int):
            Approximate user population that will be
            targeted, rounded to the nearest 100.

            This field is a member of `oneof`_ ``_reach``.
        search_term (str):
            If the request searched by location name,
            this is the location name that matched the geo
            target.

            This field is a member of `oneof`_ ``_search_term``.
        geo_target_constant (google.ads.googleads.v21.resources.types.GeoTargetConstant):
            The GeoTargetConstant result.
        geo_target_constant_parents (MutableSequence[google.ads.googleads.v21.resources.types.GeoTargetConstant]):
            The list of parents of the geo target
            constant.
    """

    locale: str = proto.Field(
        proto.STRING,
        number=6,
        optional=True,
    )
    reach: int = proto.Field(
        proto.INT64,
        number=7,
        optional=True,
    )
    search_term: str = proto.Field(
        proto.STRING,
        number=8,
        optional=True,
    )
    geo_target_constant: gagr_geo_target_constant.GeoTargetConstant = (
        proto.Field(
            proto.MESSAGE,
            number=4,
            message=gagr_geo_target_constant.GeoTargetConstant,
        )
    )
    geo_target_constant_parents: MutableSequence[
        gagr_geo_target_constant.GeoTargetConstant
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=5,
        message=gagr_geo_target_constant.GeoTargetConstant,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v22.enums.types import mobile_device_type


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "MobileDeviceConstant",
    },
)


class MobileDeviceConstant(proto.Message):
    r"""A mobile device constant.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Output only. The resource name of the mobile device
            constant. Mobile device constant resource names have the
            form:

            ``mobileDeviceConstants/{criterion_id}``
        id (int):
            Output only. The ID of the mobile device
            constant.

            This field is a member of `oneof`_ ``_id``.
        name (str):
            Output only. The name of the mobile device.

            This field is a member of `oneof`_ ``_name``.
        manufacturer_name (str):
            Output only. The manufacturer of the mobile
            device.

            This field is a member of `oneof`_ ``_manufacturer_name``.
        operating_system_name (str):
            Output only. The operating system of the
            mobile device.

            This field is a member of `oneof`_ ``_operating_system_name``.
        type_ (google.ads.googleads.v22.enums.types.MobileDeviceTypeEnum.MobileDeviceType):
            Output only. The type of mobile device.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=7,
        optional=True,
    )
    name: str = proto.Field(
        proto.STRING,
        number=8,
        optional=True,
    )
    manufacturer_name: str = proto.Field(
        proto.STRING,
        number=9,
        optional=True,
    )
    operating_system_name: str = proto.Field(
        proto.STRING,
        number=10,
        optional=True,
    )
    type_: mobile_device_type.MobileDeviceTypeEnum.MobileDeviceType = (
        proto.Field(
            proto.ENUM,
            number=6,
            enum=mobile_device_type.MobileDeviceTypeEnum.MobileDeviceType,
        )
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.errors",
    marshal="google.ads.googleads.v20",
    manifest={
        "KeywordPlanAdGroupKeywordErrorEnum",
    },
)


class KeywordPlanAdGroupKeywordErrorEnum(proto.Message):
    r"""Container for enum describing possible errors from applying
    an ad group keyword or a campaign keyword from a keyword plan.

    """

    class KeywordPlanAdGroupKeywordError(proto.Enum):
        r"""Enum describing possible errors from applying a keyword plan
        ad group keyword or keyword plan campaign keyword.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            INVALID_KEYWORD_MATCH_TYPE (2):
                A keyword or negative keyword has invalid
                match type.
            DUPLICATE_KEYWORD (3):
                A keyword or negative keyword with same text
                and match type already exists.
            KEYWORD_TEXT_TOO_LONG (4):
                Keyword or negative keyword text exceeds the
                allowed limit.
            KEYWORD_HAS_INVALID_CHARS (5):
                Keyword or negative keyword text has invalid
                characters or symbols.
            KEYWORD_HAS_TOO_MANY_WORDS (6):
                Keyword or negative keyword text has too many
                words.
            INVALID_KEYWORD_TEXT (7):
                Keyword or negative keyword has invalid text.
            NEGATIVE_KEYWORD_HAS_CPC_BID (8):
                Cpc Bid set for negative keyword.
            NEW_BMM_KEYWORDS_NOT_ALLOWED (9):
                New broad match modifier (BMM)
                KpAdGroupKeywords are not allowed.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        INVALID_KEYWORD_MATCH_TYPE = 2
        DUPLICATE_KEYWORD = 3
        KEYWORD_TEXT_TOO_LONG = 4
        KEYWORD_HAS_INVALID_CHARS = 5
        KEYWORD_HAS_TOO_MANY_WORDS = 6
        INVALID_KEYWORD_TEXT = 7
        NEGATIVE_KEYWORD_HAS_CPC_BID = 8
        NEW_BMM_KEYWORDS_NOT_ALLOWED = 9


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v20.common.types import local_services
from google.ads.googleads.v20.enums.types import (
    local_services_business_registration_check_rejection_reason,
)
from google.ads.googleads.v20.enums.types import (
    local_services_business_registration_type,
)
from google.ads.googleads.v20.enums.types import (
    local_services_insurance_rejection_reason,
)
from google.ads.googleads.v20.enums.types import (
    local_services_license_rejection_reason,
)
from google.ads.googleads.v20.enums.types import (
    local_services_verification_artifact_status,
)
from google.ads.googleads.v20.enums.types import (
    local_services_verification_artifact_type,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.resources",
    marshal="google.ads.googleads.v20",
    manifest={
        "LocalServicesVerificationArtifact",
        "BackgroundCheckVerificationArtifact",
        "InsuranceVerificationArtifact",
        "LicenseVerificationArtifact",
        "BusinessRegistrationCheckVerificationArtifact",
        "BusinessRegistrationNumber",
        "BusinessRegistrationDocument",
    },
)


class LocalServicesVerificationArtifact(proto.Message):
    r"""A local services verification resource.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the Local Services
            Verification. Local Services Verification resource names
            have the form:

            ``customers/{customer_id}/localServicesVerificationArtifacts/{verification_artifact_id}``
        id (int):
            Output only. The ID of the verification
            artifact.

            This field is a member of `oneof`_ ``_id``.
        creation_date_time (str):
            Output only. The timestamp when this
            verification artifact was created. The format is
            "YYYY-MM-DD HH:MM:SS" in the Google Ads
            account's timezone. Examples: "2018-03-05
            09:15:00" or "2018-02-01 14:34:30".
        status (google.ads.googleads.v20.enums.types.LocalServicesVerificationArtifactStatusEnum.LocalServicesVerificationArtifactStatus):
            Output only. The status of the verification
            artifact.
        artifact_type (google.ads.googleads.v20.enums.types.LocalServicesVerificationArtifactTypeEnum.LocalServicesVerificationArtifactType):
            Output only. The type of the verification
            artifact.
        background_check_verification_artifact (google.ads.googleads.v20.resources.types.BackgroundCheckVerificationArtifact):
            Output only. A background check verification
            artifact.

            This field is a member of `oneof`_ ``artifact_data``.
        insurance_verification_artifact (google.ads.googleads.v20.resources.types.InsuranceVerificationArtifact):
            Output only. An insurance verification
            artifact.

            This field is a member of `oneof`_ ``artifact_data``.
        license_verification_artifact (google.ads.googleads.v20.resources.types.LicenseVerificationArtifact):
            Output only. A license verification artifact.

            This field is a member of `oneof`_ ``artifact_data``.
        business_registration_check_verification_artifact (google.ads.googleads.v20.resources.types.BusinessRegistrationCheckVerificationArtifact):
            Output only. A business registration check
            verification artifact.

            This field is a member of `oneof`_ ``artifact_data``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=2,
        optional=True,
    )
    creation_date_time: str = proto.Field(
        proto.STRING,
        number=3,
    )
    status: (
        local_services_verification_artifact_status.LocalServicesVerificationArtifactStatusEnum.LocalServicesVerificationArtifactStatus
    ) = proto.Field(
        proto.ENUM,
        number=4,
        enum=local_services_verification_artifact_status.LocalServicesVerificationArtifactStatusEnum.LocalServicesVerificationArtifactStatus,
    )
    artifact_type: (
        local_services_verification_artifact_type.LocalServicesVerificationArtifactTypeEnum.LocalServicesVerificationArtifactType
    ) = proto.Field(
        proto.ENUM,
        number=5,
        enum=local_services_verification_artifact_type.LocalServicesVerificationArtifactTypeEnum.LocalServicesVerificationArtifactType,
    )
    background_check_verification_artifact: (
        "BackgroundCheckVerificationArtifact"
    ) = proto.Field(
        proto.MESSAGE,
        number=6,
        oneof="artifact_data",
        message="BackgroundCheckVerificationArtifact",
    )
    insurance_verification_artifact: "InsuranceVerificationArtifact" = (
        proto.Field(
            proto.MESSAGE,
            number=7,
            oneof="artifact_data",
            message="InsuranceVerificationArtifact",
        )
    )
    license_verification_artifact: "LicenseVerificationArtifact" = proto.Field(
        proto.MESSAGE,
        number=8,
        oneof="artifact_data",
        message="LicenseVerificationArtifact",
    )
    business_registration_check_verification_artifact: (
        "BusinessRegistrationCheckVerificationArtifact"
    ) = proto.Field(
        proto.MESSAGE,
        number=9,
        oneof="artifact_data",
        message="BusinessRegistrationCheckVerificationArtifact",
    )


class BackgroundCheckVerificationArtifact(proto.Message):
    r"""A proto holding information specific to local services
    background check.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        case_url (str):
            Output only. URL to access background case.

            This field is a member of `oneof`_ ``_case_url``.
        final_adjudication_date_time (str):
            Output only. The timestamp when this
            background check case result was adjudicated.
            The format is "YYYY-MM-DD HH:MM:SS" in the
            Google Ads account's timezone. Examples:
            "2018-03-05 09:15:00" or "2018-02-01 14:34:30".

            This field is a member of `oneof`_ ``_final_adjudication_date_time``.
    """

    case_url: str = proto.Field(
        proto.STRING,
        number=1,
        optional=True,
    )
    final_adjudication_date_time: str = proto.Field(
        proto.STRING,
        number=2,
        optional=True,
    )


class InsuranceVerificationArtifact(proto.Message):
    r"""A proto holding information specific to a local services
    insurance.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        amount_micros (int):
            Output only. Insurance amount. This is
            measured in "micros" of the currency mentioned
            in the insurance document.

            This field is a member of `oneof`_ ``_amount_micros``.
        rejection_reason (google.ads.googleads.v20.enums.types.LocalServicesInsuranceRejectionReasonEnum.LocalServicesInsuranceRejectionReason):
            Output only. Insurance document's rejection
            reason.

            This field is a member of `oneof`_ ``_rejection_reason``.
        insurance_document_readonly (google.ads.googleads.v20.common.types.LocalServicesDocumentReadOnly):
            Output only. The readonly field containing
            the information for an uploaded insurance
            document.

            This field is a member of `oneof`_ ``_insurance_document_readonly``.
        expiration_date_time (str):
            Output only. The timestamp when this
            insurance expires. The format is "YYYY-MM-DD
            HH:MM:SS" in the Google Ads account's timezone.
            Examples: "2018-03-05 09:15:00" or "2018-02-01
            14:34:30".

            This field is a member of `oneof`_ ``_expiration_date_time``.
    """

    amount_micros: int = proto.Field(
        proto.INT64,
        number=1,
        optional=True,
    )
    rejection_reason: (
        local_services_insurance_rejection_reason.LocalServicesInsuranceRejectionReasonEnum.LocalServicesInsuranceRejectionReason
    ) = proto.Field(
        proto.ENUM,
        number=2,
        optional=True,
        enum=local_services_insurance_rejection_reason.LocalServicesInsuranceRejectionReasonEnum.LocalServicesInsuranceRejectionReason,
    )
    insurance_document_readonly: (
        local_services.LocalServicesDocumentReadOnly
    ) = proto.Field(
        proto.MESSAGE,
        number=3,
        optional=True,
        message=local_services.LocalServicesDocumentReadOnly,
    )
    expiration_date_time: str = proto.Field(
        proto.STRING,
        number=4,
        optional=True,
    )


class LicenseVerificationArtifact(proto.Message):
    r"""A proto holding information specific to a local services
    license.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        license_type (str):
            Output only. License type / name.

            This field is a member of `oneof`_ ``_license_type``.
        license_number (str):
            Output only. License number.

            This field is a member of `oneof`_ ``_license_number``.
        licensee_first_name (str):
            Output only. First name of the licensee.

            This field is a member of `oneof`_ ``_licensee_first_name``.
        licensee_last_name (str):
            Output only. Last name of the licensee.

            This field is a member of `oneof`_ ``_licensee_last_name``.
        rejection_reason (google.ads.googleads.v20.enums.types.LocalServicesLicenseRejectionReasonEnum.LocalServicesLicenseRejectionReason):
            Output only. License rejection reason.

            This field is a member of `oneof`_ ``_rejection_reason``.
        license_document_readonly (google.ads.googleads.v20.common.types.LocalServicesDocumentReadOnly):
            Output only. The readonly field containing
            the information for an uploaded license
            document.

            This field is a member of `oneof`_ ``_license_document_readonly``.
        expiration_date_time (str):
            Output only. The timestamp when this license
            expires. The format is "YYYY-MM-DD HH:MM:SS" in
            the Google Ads account's timezone. Examples:
            "2018-03-05 09:15:00" or "2018-02-01 14:34:30".

            This field is a member of `oneof`_ ``_expiration_date_time``.
    """

    license_type: str = proto.Field(
        proto.STRING,
        number=1,
        optional=True,
    )
    license_number: str = proto.Field(
        proto.STRING,
        number=2,
        optional=True,
    )
    licensee_first_name: str = proto.Field(
        proto.STRING,
        number=3,
        optional=True,
    )
    licensee_last_name: str = proto.Field(
        proto.STRING,
        number=4,
        optional=True,
    )
    rejection_reason: (
        local_services_license_rejection_reason.LocalServicesLicenseRejectionReasonEnum.LocalServicesLicenseRejectionReason
    ) = proto.Field(
        proto.ENUM,
        number=5,
        optional=True,
        enum=local_services_license_rejection_reason.LocalServicesLicenseRejectionReasonEnum.LocalServicesLicenseRejectionReason,
    )
    license_document_readonly: local_services.LocalServicesDocumentReadOnly = (
        proto.Field(
            proto.MESSAGE,
            number=6,
            optional=True,
            message=local_services.LocalServicesDocumentReadOnly,
        )
    )
    expiration_date_time: str = proto.Field(
        proto.STRING,
        number=7,
        optional=True,
    )


class BusinessRegistrationCheckVerificationArtifact(proto.Message):
    r"""A proto holding information specific to a local services
    business registration check.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        registration_type (google.ads.googleads.v20.enums.types.LocalServicesBusinessRegistrationTypeEnum.LocalServicesBusinessRegistrationType):
            Output only. The type of business
            registration check (number, document).

            This field is a member of `oneof`_ ``_registration_type``.
        check_id (str):
            Output only. The id of the check, such as vat_tax_id,
            representing "VAT Tax ID" requirement.

            This field is a member of `oneof`_ ``_check_id``.
        rejection_reason (google.ads.googleads.v20.enums.types.LocalServicesBusinessRegistrationCheckRejectionReasonEnum.LocalServicesBusinessRegistrationCheckRejectionReason):
            Output only. Registration document rejection
            reason.

            This field is a member of `oneof`_ ``_rejection_reason``.
        registration_number (google.ads.googleads.v20.resources.types.BusinessRegistrationNumber):
            Output only. Message storing government
            issued number for the business.

            This field is a member of `oneof`_ ``business_registration``.
        registration_document (google.ads.googleads.v20.resources.types.BusinessRegistrationDocument):
            Output only. Message storing document info
            for the business.

            This field is a member of `oneof`_ ``business_registration``.
    """

    registration_type: (
        local_services_business_registration_type.LocalServicesBusinessRegistrationTypeEnum.LocalServicesBusinessRegistrationType
    ) = proto.Field(
        proto.ENUM,
        number=3,
        optional=True,
        enum=local_services_business_registration_type.LocalServicesBusinessRegistrationTypeEnum.LocalServicesBusinessRegistrationType,
    )
    check_id: str = proto.Field(
        proto.STRING,
        number=4,
        optional=True,
    )
    rejection_reason: (
        local_services_business_registration_check_rejection_reason.LocalServicesBusinessRegistrationCheckRejectionReasonEnum.LocalServicesBusinessRegistrationCheckRejectionReason
    ) = proto.Field(
        proto.ENUM,
        number=5,
        optional=True,
        enum=local_services_business_registration_check_rejection_reason.LocalServicesBusinessRegistrationCheckRejectionReasonEnum.LocalServicesBusinessRegistrationCheckRejectionReason,
    )
    registration_number: "BusinessRegistrationNumber" = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="business_registration",
        message="BusinessRegistrationNumber",
    )
    registration_document: "BusinessRegistrationDocument" = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="business_registration",
        message="BusinessRegistrationDocument",
    )


class BusinessRegistrationNumber(proto.Message):
    r"""A proto holding information specific to a local services
    business registration number.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        number (str):
            Output only. Government-issued number for the
            business.

            This field is a member of `oneof`_ ``_number``.
    """

    number: str = proto.Field(
        proto.STRING,
        number=1,
        optional=True,
    )


class BusinessRegistrationDocument(proto.Message):
    r"""A proto holding information specific to a local services
    business registration document.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        document_readonly (google.ads.googleads.v20.common.types.LocalServicesDocumentReadOnly):
            Output only. The readonly field containing
            the information for an uploaded business
            registration document.

            This field is a member of `oneof`_ ``_document_readonly``.
    """

    document_readonly: local_services.LocalServicesDocumentReadOnly = (
        proto.Field(
            proto.MESSAGE,
            number=1,
            optional=True,
            message=local_services.LocalServicesDocumentReadOnly,
        )
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.enums",
    marshal="google.ads.googleads.v19",
    manifest={
        "ConversionValueRulePrimaryDimensionEnum",
    },
)


class ConversionValueRulePrimaryDimensionEnum(proto.Message):
    r"""Container for enum describing value rule primary dimension
    for stats.

    """

    class ConversionValueRulePrimaryDimension(proto.Enum):
        r"""Identifies the primary dimension for conversion value rule
        stats.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            NO_RULE_APPLIED (2):
                For no-value-rule-applied conversions after
                value rule is enabled.
            ORIGINAL (3):
                Below are for value-rule-applied conversions:
                The original stats.
            NEW_VS_RETURNING_USER (4):
                When a new or returning customer condition is
                satisfied.
            GEO_LOCATION (5):
                When a query-time Geo location condition is
                satisfied.
            DEVICE (6):
                When a query-time browsing device condition
                is satisfied.
            AUDIENCE (7):
                When a query-time audience condition is
                satisfied.
            MULTIPLE (8):
                When multiple rules are applied.
            ITINERARY (9):
                When a query-time itinerary condition is
                satisfied.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        NO_RULE_APPLIED = 2
        ORIGINAL = 3
        NEW_VS_RETURNING_USER = 4
        GEO_LOCATION = 5
        DEVICE = 6
        AUDIENCE = 7
        MULTIPLE = 8
        ITINERARY = 9


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.errors",
    marshal="google.ads.googleads.v21",
    manifest={
        "AssetGroupErrorEnum",
    },
)


class AssetGroupErrorEnum(proto.Message):
    r"""Container for enum describing possible asset group errors."""

    class AssetGroupError(proto.Enum):
        r"""Enum describing possible asset group errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            DUPLICATE_NAME (2):
                Each asset group in a single campaign must
                have a unique name.
            CANNOT_ADD_ASSET_GROUP_FOR_CAMPAIGN_TYPE (3):
                Cannot add asset group for the campaign type.
            NOT_ENOUGH_HEADLINE_ASSET (4):
                Not enough headline asset for a valid asset
                group.
            NOT_ENOUGH_LONG_HEADLINE_ASSET (5):
                Not enough long headline asset for a valid
                asset group.
            NOT_ENOUGH_DESCRIPTION_ASSET (6):
                Not enough description headline asset for a
                valid asset group.
            NOT_ENOUGH_BUSINESS_NAME_ASSET (7):
                Not enough business name asset for a valid
                asset group.
            NOT_ENOUGH_MARKETING_IMAGE_ASSET (8):
                Not enough marketing image asset for a valid
                asset group.
            NOT_ENOUGH_SQUARE_MARKETING_IMAGE_ASSET (9):
                Not enough square marketing image asset for a
                valid asset group.
            NOT_ENOUGH_LOGO_ASSET (10):
                Not enough logo asset for a valid asset
                group.
            FINAL_URL_SHOPPING_MERCHANT_HOME_PAGE_URL_DOMAINS_DIFFER (11):
                Final url and shopping merchant url does not
                have the same domain.
            PATH1_REQUIRED_WHEN_PATH2_IS_SET (12):
                Path1 required when path2 is set.
            SHORT_DESCRIPTION_REQUIRED (13):
                At least one short description asset is
                required for a valid asset group.
            FINAL_URL_REQUIRED (14):
                Final url field is required for asset group.
            FINAL_URL_CONTAINS_INVALID_DOMAIN_NAME (15):
                Final url contains invalid domain name.
            AD_CUSTOMIZER_NOT_SUPPORTED (16):
                Ad customizers are not supported in asset
                group's text field.
            CANNOT_MUTATE_ASSET_GROUP_FOR_REMOVED_CAMPAIGN (17):
                Cannot mutate asset group for campaign with
                removed status.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        DUPLICATE_NAME = 2
        CANNOT_ADD_ASSET_GROUP_FOR_CAMPAIGN_TYPE = 3
        NOT_ENOUGH_HEADLINE_ASSET = 4
        NOT_ENOUGH_LONG_HEADLINE_ASSET = 5
        NOT_ENOUGH_DESCRIPTION_ASSET = 6
        NOT_ENOUGH_BUSINESS_NAME_ASSET = 7
        NOT_ENOUGH_MARKETING_IMAGE_ASSET = 8
        NOT_ENOUGH_SQUARE_MARKETING_IMAGE_ASSET = 9
        NOT_ENOUGH_LOGO_ASSET = 10
        FINAL_URL_SHOPPING_MERCHANT_HOME_PAGE_URL_DOMAINS_DIFFER = 11
        PATH1_REQUIRED_WHEN_PATH2_IS_SET = 12
        SHORT_DESCRIPTION_REQUIRED = 13
        FINAL_URL_REQUIRED = 14
        FINAL_URL_CONTAINS_INVALID_DOMAIN_NAME = 15
        AD_CUSTOMIZER_NOT_SUPPORTED = 16
        CANNOT_MUTATE_ASSET_GROUP_FOR_REMOVED_CAMPAIGN = 17


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.errors",
    marshal="google.ads.googleads.v21",
    manifest={
        "AdGroupCriterionErrorEnum",
    },
)


class AdGroupCriterionErrorEnum(proto.Message):
    r"""Container for enum describing possible ad group criterion
    errors.

    """

    class AdGroupCriterionError(proto.Enum):
        r"""Enum describing possible ad group criterion errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            AD_GROUP_CRITERION_LABEL_DOES_NOT_EXIST (2):
                No link found between the AdGroupCriterion
                and the label.
            AD_GROUP_CRITERION_LABEL_ALREADY_EXISTS (3):
                The label has already been attached to the
                AdGroupCriterion.
            CANNOT_ADD_LABEL_TO_NEGATIVE_CRITERION (4):
                Negative AdGroupCriterion cannot have labels.
            TOO_MANY_OPERATIONS (5):
                Too many operations for a single call.
            CANT_UPDATE_NEGATIVE (6):
                Negative ad group criteria are not
                updateable.
            CONCRETE_TYPE_REQUIRED (7):
                Concrete type of criterion (keyword v.s.
                placement) is required for ADD and SET
                operations.
            BID_INCOMPATIBLE_WITH_ADGROUP (8):
                Bid is incompatible with ad group's bidding
                settings.
            CANNOT_TARGET_AND_EXCLUDE (9):
                Cannot target and exclude the same criterion
                at once.
            ILLEGAL_URL (10):
                The URL of a placement is invalid.
            INVALID_KEYWORD_TEXT (11):
                Keyword text was invalid.
            INVALID_DESTINATION_URL (12):
                Destination URL was invalid.
            MISSING_DESTINATION_URL_TAG (13):
                The destination url must contain at least one
                tag (for example, {lpurl})
            KEYWORD_LEVEL_BID_NOT_SUPPORTED_FOR_MANUALCPM (14):
                Keyword-level cpm bid is not supported
            INVALID_USER_STATUS (15):
                For example, cannot add a biddable ad group
                criterion that had been removed.
            CANNOT_ADD_CRITERIA_TYPE (16):
                Criteria type cannot be targeted for the ad
                group. Either the account is restricted to
                keywords only, the criteria type is incompatible
                with the campaign's bidding strategy, or the
                criteria type can only be applied to campaigns.
            CANNOT_EXCLUDE_CRITERIA_TYPE (17):
                Criteria type cannot be excluded for the ad
                group. Refer to the documentation for a specific
                criterion to check if it is excludable.
            CAMPAIGN_TYPE_NOT_COMPATIBLE_WITH_PARTIAL_FAILURE (27):
                Partial failure is not supported for shopping
                campaign mutate operations.
            OPERATIONS_FOR_TOO_MANY_SHOPPING_ADGROUPS (28):
                Operations in the mutate request changes too
                many shopping ad groups. Split requests for
                multiple shopping ad groups across multiple
                requests.
            CANNOT_MODIFY_URL_FIELDS_WITH_DUPLICATE_ELEMENTS (29):
                Not allowed to modify url fields of an ad
                group criterion if there are duplicate elements
                for that ad group criterion in the request.
            CANNOT_SET_WITHOUT_FINAL_URLS (30):
                Cannot set url fields without also setting
                final urls.
            CANNOT_CLEAR_FINAL_URLS_IF_FINAL_MOBILE_URLS_EXIST (31):
                Cannot clear final urls if final mobile urls
                exist.
            CANNOT_CLEAR_FINAL_URLS_IF_FINAL_APP_URLS_EXIST (32):
                Cannot clear final urls if final app urls
                exist.
            CANNOT_CLEAR_FINAL_URLS_IF_TRACKING_URL_TEMPLATE_EXISTS (33):
                Cannot clear final urls if tracking url
                template exists.
            CANNOT_CLEAR_FINAL_URLS_IF_URL_CUSTOM_PARAMETERS_EXIST (34):
                Cannot clear final urls if url custom
                parameters exist.
            CANNOT_SET_BOTH_DESTINATION_URL_AND_FINAL_URLS (35):
                Cannot set both destination url and final
                urls.
            CANNOT_SET_BOTH_DESTINATION_URL_AND_TRACKING_URL_TEMPLATE (36):
                Cannot set both destination url and tracking
                url template.
            FINAL_URLS_NOT_SUPPORTED_FOR_CRITERION_TYPE (37):
                Final urls are not supported for this
                criterion type.
            FINAL_MOBILE_URLS_NOT_SUPPORTED_FOR_CRITERION_TYPE (38):
                Final mobile urls are not supported for this
                criterion type.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        AD_GROUP_CRITERION_LABEL_DOES_NOT_EXIST = 2
        AD_GROUP_CRITERION_LABEL_ALREADY_EXISTS = 3
        CANNOT_ADD_LABEL_TO_NEGATIVE_CRITERION = 4
        TOO_MANY_OPERATIONS = 5
        CANT_UPDATE_NEGATIVE = 6
        CONCRETE_TYPE_REQUIRED = 7
        BID_INCOMPATIBLE_WITH_ADGROUP = 8
        CANNOT_TARGET_AND_EXCLUDE = 9
        ILLEGAL_URL = 10
        INVALID_KEYWORD_TEXT = 11
        INVALID_DESTINATION_URL = 12
        MISSING_DESTINATION_URL_TAG = 13
        KEYWORD_LEVEL_BID_NOT_SUPPORTED_FOR_MANUALCPM = 14
        INVALID_USER_STATUS = 15
        CANNOT_ADD_CRITERIA_TYPE = 16
        CANNOT_EXCLUDE_CRITERIA_TYPE = 17
        CAMPAIGN_TYPE_NOT_COMPATIBLE_WITH_PARTIAL_FAILURE = 27
        OPERATIONS_FOR_TOO_MANY_SHOPPING_ADGROUPS = 28
        CANNOT_MODIFY_URL_FIELDS_WITH_DUPLICATE_ELEMENTS = 29
        CANNOT_SET_WITHOUT_FINAL_URLS = 30
        CANNOT_CLEAR_FINAL_URLS_IF_FINAL_MOBILE_URLS_EXIST = 31
        CANNOT_CLEAR_FINAL_URLS_IF_FINAL_APP_URLS_EXIST = 32
        CANNOT_CLEAR_FINAL_URLS_IF_TRACKING_URL_TEMPLATE_EXISTS = 33
        CANNOT_CLEAR_FINAL_URLS_IF_URL_CUSTOM_PARAMETERS_EXIST = 34
        CANNOT_SET_BOTH_DESTINATION_URL_AND_FINAL_URLS = 35
        CANNOT_SET_BOTH_DESTINATION_URL_AND_TRACKING_URL_TEMPLATE = 36
        FINAL_URLS_NOT_SUPPORTED_FOR_CRITERION_TYPE = 37
        FINAL_MOBILE_URLS_NOT_SUPPORTED_FOR_CRITERION_TYPE = 38


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v20.enums.types import (
    user_list_customer_type_category,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.resources",
    marshal="google.ads.googleads.v20",
    manifest={
        "UserListCustomerType",
    },
)


class UserListCustomerType(proto.Message):
    r"""A user list customer type

    Attributes:
        resource_name (str):
            Immutable. The resource name of the user list customer type
            User list customer type resource names have the form:
            ``customers/{customer_id}/userListCustomerTypes/{user_list_id}~{customer_type_category}``
        user_list (str):
            Immutable. The resource name for the user
            list this user list customer type is associated
            with
        customer_type_category (google.ads.googleads.v20.enums.types.UserListCustomerTypeCategoryEnum.UserListCustomerTypeCategory):
            Immutable. The user list customer type
            category
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    user_list: str = proto.Field(
        proto.STRING,
        number=2,
    )
    customer_type_category: (
        user_list_customer_type_category.UserListCustomerTypeCategoryEnum.UserListCustomerTypeCategory
    ) = proto.Field(
        proto.ENUM,
        number=3,
        enum=user_list_customer_type_category.UserListCustomerTypeCategoryEnum.UserListCustomerTypeCategory,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v19.enums.types import budget_delivery_method
from google.ads.googleads.v19.enums.types import budget_period
from google.ads.googleads.v19.enums.types import budget_status
from google.ads.googleads.v19.enums.types import budget_type


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.resources",
    marshal="google.ads.googleads.v19",
    manifest={
        "CampaignBudget",
    },
)


class CampaignBudget(proto.Message):
    r"""A campaign budget.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the campaign budget.
            Campaign budget resource names have the form:

            ``customers/{customer_id}/campaignBudgets/{campaign_budget_id}``
        id (int):
            Output only. The ID of the campaign budget.

            A campaign budget is created using the
            CampaignBudgetService create operation and is
            assigned a budget ID. A budget ID can be shared
            across different campaigns; the system will then
            allocate the campaign budget among different
            campaigns to get optimum results.

            This field is a member of `oneof`_ ``_id``.
        name (str):
            The name of the campaign budget.

            When creating a campaign budget through
            CampaignBudgetService, every explicitly shared
            campaign budget must have a non-null, non-empty
            name. Campaign budgets that are not explicitly
            shared derive their name from the attached
            campaign's name.

            The length of this string must be between 1 and
            255, inclusive, in UTF-8 bytes, (trimmed).

            This field is a member of `oneof`_ ``_name``.
        amount_micros (int):
            The amount of the budget, in the local
            currency for the account. Amount is specified in
            micros, where one million is equivalent to one
            currency unit. Monthly spend is capped at 30.4
            times this amount.

            This field is a member of `oneof`_ ``_amount_micros``.
        total_amount_micros (int):
            The lifetime amount of the budget, in the
            local currency for the account. Amount is
            specified in micros, where one million is
            equivalent to one currency unit.

            This field is a member of `oneof`_ ``_total_amount_micros``.
        status (google.ads.googleads.v19.enums.types.BudgetStatusEnum.BudgetStatus):
            Output only. The status of this campaign
            budget. This field is read-only.
        delivery_method (google.ads.googleads.v19.enums.types.BudgetDeliveryMethodEnum.BudgetDeliveryMethod):
            The delivery method that determines the rate
            at which the campaign budget is spent.

            Defaults to STANDARD if unspecified in a create
            operation.
        explicitly_shared (bool):
            Specifies whether the budget is explicitly
            shared. Defaults to true if unspecified in a
            create operation.

            If true, the budget was created with the purpose
            of sharing across one or more campaigns.

            If false, the budget was created with the
            intention of only being used with a single
            campaign. The budget's name and status will stay
            in sync with the campaign's name and status.
            Attempting to share the budget with a second
            campaign will result in an error.

            A non-shared budget can become an explicitly
            shared. The same operation must also assign the
            budget a name.

            A shared campaign budget can never become
            non-shared.

            This field is a member of `oneof`_ ``_explicitly_shared``.
        reference_count (int):
            Output only. The number of campaigns actively
            using the budget.
            This field is read-only.

            This field is a member of `oneof`_ ``_reference_count``.
        has_recommended_budget (bool):
            Output only. Indicates whether there is a
            recommended budget for this campaign budget.

            This field is read-only.

            This field is a member of `oneof`_ ``_has_recommended_budget``.
        recommended_budget_amount_micros (int):
            Output only. The recommended budget amount.
            If no recommendation is available, this will be
            set to the budget amount. Amount is specified in
            micros, where one million is equivalent to one
            currency unit.

            This field is read-only.

            This field is a member of `oneof`_ ``_recommended_budget_amount_micros``.
        period (google.ads.googleads.v19.enums.types.BudgetPeriodEnum.BudgetPeriod):
            Immutable. Period over which to spend the
            budget. Defaults to DAILY if not specified.
        recommended_budget_estimated_change_weekly_clicks (int):
            Output only. The estimated change in weekly
            clicks if the recommended budget is applied.

            This field is read-only.

            This field is a member of `oneof`_ ``_recommended_budget_estimated_change_weekly_clicks``.
        recommended_budget_estimated_change_weekly_cost_micros (int):
            Output only. The estimated change in weekly
            cost in micros if the recommended budget is
            applied. One million is equivalent to one
            currency unit.

            This field is read-only.

            This field is a member of `oneof`_ ``_recommended_budget_estimated_change_weekly_cost_micros``.
        recommended_budget_estimated_change_weekly_interactions (int):
            Output only. The estimated change in weekly
            interactions if the recommended budget is
            applied.

            This field is read-only.

            This field is a member of `oneof`_ ``_recommended_budget_estimated_change_weekly_interactions``.
        recommended_budget_estimated_change_weekly_views (int):
            Output only. The estimated change in weekly
            views if the recommended budget is applied.

            This field is read-only.

            This field is a member of `oneof`_ ``_recommended_budget_estimated_change_weekly_views``.
        type_ (google.ads.googleads.v19.enums.types.BudgetTypeEnum.BudgetType):
            Immutable. The type of the campaign budget.
        aligned_bidding_strategy_id (int):
            ID of the portfolio bidding strategy that
            this shared campaign budget is aligned with.
            When a bidding strategy and a campaign budget
            are aligned, they are attached to the same set
            of campaigns. After a campaign budget is aligned
            with a bidding strategy, campaigns that are
            added to the campaign budget must also use the
            aligned bidding strategy.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=19,
        optional=True,
    )
    name: str = proto.Field(
        proto.STRING,
        number=20,
        optional=True,
    )
    amount_micros: int = proto.Field(
        proto.INT64,
        number=21,
        optional=True,
    )
    total_amount_micros: int = proto.Field(
        proto.INT64,
        number=22,
        optional=True,
    )
    status: budget_status.BudgetStatusEnum.BudgetStatus = proto.Field(
        proto.ENUM,
        number=6,
        enum=budget_status.BudgetStatusEnum.BudgetStatus,
    )
    delivery_method: (
        budget_delivery_method.BudgetDeliveryMethodEnum.BudgetDeliveryMethod
    ) = proto.Field(
        proto.ENUM,
        number=7,
        enum=budget_delivery_method.BudgetDeliveryMethodEnum.BudgetDeliveryMethod,
    )
    explicitly_shared: bool = proto.Field(
        proto.BOOL,
        number=23,
        optional=True,
    )
    reference_count: int = proto.Field(
        proto.INT64,
        number=24,
        optional=True,
    )
    has_recommended_budget: bool = proto.Field(
        proto.BOOL,
        number=25,
        optional=True,
    )
    recommended_budget_amount_micros: int = proto.Field(
        proto.INT64,
        number=26,
        optional=True,
    )
    period: budget_period.BudgetPeriodEnum.BudgetPeriod = proto.Field(
        proto.ENUM,
        number=13,
        enum=budget_period.BudgetPeriodEnum.BudgetPeriod,
    )
    recommended_budget_estimated_change_weekly_clicks: int = proto.Field(
        proto.INT64,
        number=27,
        optional=True,
    )
    recommended_budget_estimated_change_weekly_cost_micros: int = proto.Field(
        proto.INT64,
        number=28,
        optional=True,
    )
    recommended_budget_estimated_change_weekly_interactions: int = proto.Field(
        proto.INT64,
        number=29,
        optional=True,
    )
    recommended_budget_estimated_change_weekly_views: int = proto.Field(
        proto.INT64,
        number=30,
        optional=True,
    )
    type_: budget_type.BudgetTypeEnum.BudgetType = proto.Field(
        proto.ENUM,
        number=18,
        enum=budget_type.BudgetTypeEnum.BudgetType,
    )
    aligned_bidding_strategy_id: int = proto.Field(
        proto.INT64,
        number=31,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.enums",
    marshal="google.ads.googleads.v20",
    manifest={
        "BiddingStrategySystemStatusEnum",
    },
)


class BiddingStrategySystemStatusEnum(proto.Message):
    r"""Message describing BiddingStrategy system statuses."""

    class BiddingStrategySystemStatus(proto.Enum):
        r"""The possible system statuses of a BiddingStrategy.

        Values:
            UNSPECIFIED (0):
                Signals that an unexpected error occurred,
                for example, no bidding strategy type was found,
                or no status information was found.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            ENABLED (2):
                The bid strategy is active, and AdWords
                cannot find any specific issues with the
                strategy.
            LEARNING_NEW (3):
                The bid strategy is learning because it has
                been recently created or recently reactivated.
            LEARNING_SETTING_CHANGE (4):
                The bid strategy is learning because of a
                recent setting change.
            LEARNING_BUDGET_CHANGE (5):
                The bid strategy is learning because of a
                recent budget change.
            LEARNING_COMPOSITION_CHANGE (6):
                The bid strategy is learning because of
                recent change in number of campaigns, ad groups
                or keywords attached to it.
            LEARNING_CONVERSION_TYPE_CHANGE (7):
                The bid strategy depends on conversion
                reporting and the customer recently modified
                conversion types that were relevant to the bid
                strategy.
            LEARNING_CONVERSION_SETTING_CHANGE (8):
                The bid strategy depends on conversion
                reporting and the customer recently changed
                their conversion settings.
            LIMITED_BY_CPC_BID_CEILING (9):
                The bid strategy is limited by its bid
                ceiling.
            LIMITED_BY_CPC_BID_FLOOR (10):
                The bid strategy is limited by its bid floor.
            LIMITED_BY_DATA (11):
                The bid strategy is limited because there was
                not enough conversion traffic over the past
                weeks.
            LIMITED_BY_BUDGET (12):
                A significant fraction of keywords in this
                bid strategy are limited by budget.
            LIMITED_BY_LOW_PRIORITY_SPEND (13):
                The bid strategy cannot reach its target
                spend because its spend has been de-prioritized.
            LIMITED_BY_LOW_QUALITY (14):
                A significant fraction of keywords in this
                bid strategy have a low Quality Score.
            LIMITED_BY_INVENTORY (15):
                The bid strategy cannot fully spend its
                budget because of narrow targeting.
            MISCONFIGURED_ZERO_ELIGIBILITY (16):
                Missing conversion tracking (no pings
                present) and/or remarketing lists for SSC.
            MISCONFIGURED_CONVERSION_TYPES (17):
                The bid strategy depends on conversion
                reporting and the customer is lacking conversion
                types that might be reported against this
                strategy.
            MISCONFIGURED_CONVERSION_SETTINGS (18):
                The bid strategy depends on conversion
                reporting and the customer's conversion settings
                are misconfigured.
            MISCONFIGURED_SHARED_BUDGET (19):
                There are campaigns outside the bid strategy
                that share budgets with campaigns included in
                the strategy.
            MISCONFIGURED_STRATEGY_TYPE (20):
                The campaign has an invalid strategy type and
                is not serving.
            PAUSED (21):
                The bid strategy is not active. Either there
                are no active campaigns, ad groups or keywords
                attached to the bid strategy. Or there are no
                active budgets connected to the bid strategy.
            UNAVAILABLE (22):
                This bid strategy currently does not support
                status reporting.
            MULTIPLE_LEARNING (23):
                There were multiple LEARNING_\* system statuses for this bid
                strategy during the time in question.
            MULTIPLE_LIMITED (24):
                There were multiple LIMITED_\* system statuses for this bid
                strategy during the time in question.
            MULTIPLE_MISCONFIGURED (25):
                There were multiple MISCONFIGURED_\* system statuses for
                this bid strategy during the time in question.
            MULTIPLE (26):
                There were multiple system statuses for this
                bid strategy during the time in question.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        ENABLED = 2
        LEARNING_NEW = 3
        LEARNING_SETTING_CHANGE = 4
        LEARNING_BUDGET_CHANGE = 5
        LEARNING_COMPOSITION_CHANGE = 6
        LEARNING_CONVERSION_TYPE_CHANGE = 7
        LEARNING_CONVERSION_SETTING_CHANGE = 8
        LIMITED_BY_CPC_BID_CEILING = 9
        LIMITED_BY_CPC_BID_FLOOR = 10
        LIMITED_BY_DATA = 11
        LIMITED_BY_BUDGET = 12
        LIMITED_BY_LOW_PRIORITY_SPEND = 13
        LIMITED_BY_LOW_QUALITY = 14
        LIMITED_BY_INVENTORY = 15
        MISCONFIGURED_ZERO_ELIGIBILITY = 16
        MISCONFIGURED_CONVERSION_TYPES = 17
        MISCONFIGURED_CONVERSION_SETTINGS = 18
        MISCONFIGURED_SHARED_BUDGET = 19
        MISCONFIGURED_STRATEGY_TYPE = 20
        PAUSED = 21
        UNAVAILABLE = 22
        MULTIPLE_LEARNING = 23
        MULTIPLE_LIMITED = 24
        MULTIPLE_MISCONFIGURED = 25
        MULTIPLE = 26


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2018 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This example creates a standard shopping product ad.

In the process of creating a standard shopping campaign and a shopping product
ad group are also created.

Prerequisite: You need to have access to a Merchant Center account. You can find
instructions to create a Merchant Center account here:
https://support.google.com/merchants/answer/188924.

This account must be linked to your Google Ads account.
"""


import argparse
import sys
import uuid

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.resources.types.ad_group import AdGroup
from google.ads.googleads.v22.resources.types.ad_group_ad import AdGroupAd
from google.ads.googleads.v22.resources.types.ad_group_criterion import (
    AdGroupCriterion,
)
from google.ads.googleads.v22.resources.types.campaign import Campaign
from google.ads.googleads.v22.resources.types.campaign_budget import (
    CampaignBudget,
)
from google.ads.googleads.v22.services.services.ad_group_ad_service import (
    AdGroupAdServiceClient,
)
from google.ads.googleads.v22.services.services.ad_group_criterion_service import (
    AdGroupCriterionServiceClient,
)
from google.ads.googleads.v22.services.services.ad_group_service import (
    AdGroupServiceClient,
)
from google.ads.googleads.v22.services.services.campaign_budget_service import (
    CampaignBudgetServiceClient,
)
from google.ads.googleads.v22.services.services.campaign_service import (
    CampaignServiceClient,
)
from google.ads.googleads.v22.services.types.ad_group_ad_service import (
    AdGroupAdOperation,
)
from google.ads.googleads.v22.services.types.ad_group_criterion_service import (
    AdGroupCriterionOperation,
)
from google.ads.googleads.v22.services.types.ad_group_service import (
    AdGroupOperation,
)
from google.ads.googleads.v22.services.types.campaign_budget_service import (
    CampaignBudgetOperation,
)
from google.ads.googleads.v22.services.types.campaign_service import (
    CampaignOperation,
)


def main(
    client: GoogleAdsClient,
    customer_id: str,
    merchant_center_account_id: int,
    create_default_listing_group: bool,
) -> None:
    # Creates a budget to be used by the campaign that will be created below.
    budget_resource_name: str = add_campaign_budget(client, customer_id)

    # Create a standard shopping campaign.
    campaign_resource_name: str = add_standard_shopping_campaign(
        client, customer_id, budget_resource_name, merchant_center_account_id
    )

    # Create a shopping product ad group.
    ad_group_resource_name: str = add_shopping_product_ad_group(
        client, customer_id, campaign_resource_name
    )

    # Create a shopping product ad group ad.
    add_shopping_product_ad_group_ad(
        client, customer_id, ad_group_resource_name
    )

    if create_default_listing_group:
        # Creates an ad group criterion containing a listing group.
        # This will be the listing group tree for 'All products' and will
        # contain a single biddable unit node.
        add_default_shopping_listing_group(
            client, customer_id, ad_group_resource_name
        )


def add_campaign_budget(client: GoogleAdsClient, customer_id: str) -> str:
    """Creates a new campaign budget in the specified client account."""
    campaign_budget_service: CampaignBudgetServiceClient = client.get_service(
        "CampaignBudgetService"
    )

    # Create a budget, which can be shared by multiple campaigns.
    campaign_budget_operation: CampaignBudgetOperation = client.get_type(
        "CampaignBudgetOperation"
    )
    campaign_budget: CampaignBudget = campaign_budget_operation.create
    campaign_budget.name = f"Interplanetary Budget {uuid.uuid4()}"
    campaign_budget.delivery_method = (
        client.enums.BudgetDeliveryMethodEnum.STANDARD
    )
    campaign_budget.amount_micros = 500000

    # Add budget.
    campaign_budget_response = campaign_budget_service.mutate_campaign_budgets(
        customer_id=customer_id, operations=[campaign_budget_operation]
    )

    budget_resource_name: str = campaign_budget_response.results[
        0
    ].resource_name

    print(f"Added a budget with resource name: '{budget_resource_name}'")

    return budget_resource_name


# [START add_shopping_product_ad]
def add_shopping_product_ad_group_ad(
    client: GoogleAdsClient,
    customer_id: str,
    ad_group_resource_name: str,
) -> str:
    """Creates a new shopping product ad group ad in the specified ad group."""
    ad_group_ad_service: AdGroupAdServiceClient = client.get_service(
        "AdGroupAdService"
    )

    # Creates a new ad group ad and sets the product ad to it.
    ad_group_ad_operation: AdGroupAdOperation = client.get_type(
        "AdGroupAdOperation"
    )
    ad_group_ad: AdGroupAd = ad_group_ad_operation.create
    ad_group_ad.ad_group = ad_group_resource_name
    ad_group_ad.status = client.enums.AdGroupAdStatusEnum.PAUSED
    # The Ad object itself is not directly manipulated for Shopping Product Ads.
    # Instead, we copy ShoppingProductAdInfo into the ad's shopping_product_ad field.
    client.copy_from(
        ad_group_ad.ad.shopping_product_ad,
        client.get_type("ShoppingProductAdInfo"),
    )

    # Add the ad group ad.
    ad_group_ad_response = ad_group_ad_service.mutate_ad_group_ads(
        customer_id=customer_id, operations=[ad_group_ad_operation]
    )

    ad_group_ad_resource_name: str = ad_group_ad_response.results[
        0
    ].resource_name

    print(
        f"Created shopping product ad group ad '{ad_group_ad_resource_name}'."
    )
    # [END add_shopping_product_ad]


# [START add_shopping_product_ad_1]
def add_shopping_product_ad_group(
    client: GoogleAdsClient,
    customer_id: str,
    campaign_resource_name: str,
) -> str:
    """Creates a new shopping product ad group in the specified campaign."""
    ad_group_service: AdGroupServiceClient = client.get_service(
        "AdGroupService"
    )

    # Create ad group.
    ad_group_operation: AdGroupOperation = client.get_type("AdGroupOperation")
    ad_group: AdGroup = ad_group_operation.create
    ad_group.name = f"Earth to Mars cruise {uuid.uuid4()}"
    ad_group.status = client.enums.AdGroupStatusEnum.ENABLED
    ad_group.campaign = campaign_resource_name
    # Sets the ad group type to SHOPPING_PRODUCT_ADS. This is the only value
    # possible for ad groups that contain shopping product ads.
    ad_group.type_ = client.enums.AdGroupTypeEnum.SHOPPING_PRODUCT_ADS
    ad_group.cpc_bid_micros = 10000000

    # Add the ad group.
    ad_group_response = ad_group_service.mutate_ad_groups(
        customer_id=customer_id, operations=[ad_group_operation]
    )

    ad_group_resource_name: str = ad_group_response.results[0].resource_name

    print(
        "Added a product shopping ad group with resource name "
        f"'{ad_group_resource_name}'."
    )

    return ad_group_resource_name
    # [END add_shopping_product_ad_1]


# [START add_shopping_product_ad_2]
def add_standard_shopping_campaign(
    client: GoogleAdsClient,
    customer_id: str,
    budget_resource_name: str,
    merchant_center_account_id: int,
) -> str:
    """Creates a new standard shopping campaign in the specified client account."""
    campaign_service: CampaignServiceClient = client.get_service(
        "CampaignService"
    )

    # Create standard shopping campaign.
    campaign_operation: CampaignOperation = client.get_type("CampaignOperation")
    campaign: Campaign = campaign_operation.create
    campaign.name = f"Interplanetary Cruise Campaign {uuid.uuid4()}"

    # Configures settings related to shopping campaigns including advertising
    # channel type and shopping setting.
    campaign.advertising_channel_type = (
        client.enums.AdvertisingChannelTypeEnum.SHOPPING
    )
    campaign.shopping_setting.merchant_id = merchant_center_account_id

    # Sets the priority of the campaign. Higher numbers take priority over lower
    # numbers. For standard shopping campaigns, allowed values are between 0 and
    # 2, inclusive.
    campaign.shopping_setting.campaign_priority = 0

    # Enables local inventory ads for this campaign.
    campaign.shopping_setting.enable_local = True

    # Recommendation: Set the campaign to PAUSED when creating it to prevent the
    # ads from immediately serving. Set to ENABLED once you've added targeting
    # and the ads are ready to serve.
    campaign.status = client.enums.CampaignStatusEnum.PAUSED

    # Sets the bidding strategy to Manual CPC.
    # Recommendation: Use one of the automated bidding strategies for Shopping
    # campaigns to help you optimize your advertising spend. More information
    # can be found here: https://support.google.com/google-ads/answer/6309029
    campaign.manual_cpc = client.get_type("ManualCpc")

    # Sets the budget.
    campaign.campaign_budget = budget_resource_name

    # Declare whether or not this campaign serves political ads targeting the
    # EU. Valid values are:
    #   CONTAINS_EU_POLITICAL_ADVERTISING
    #   DOES_NOT_CONTAIN_EU_POLITICAL_ADVERTISING
    campaign.contains_eu_political_advertising = (
        client.enums.EuPoliticalAdvertisingStatusEnum.DOES_NOT_CONTAIN_EU_POLITICAL_ADVERTISING
    )

    # Add the campaign.
    campaign_response = campaign_service.mutate_campaigns(
        customer_id=customer_id, operations=[campaign_operation]
    )

    campaign_resource_name: str = campaign_response.results[0].resource_name

    print(
        "Added a standard shopping campaign with resource name "
        f"'{campaign_resource_name}'."
    )

    return campaign_resource_name
    # [END add_shopping_product_ad_2]


def add_default_shopping_listing_group(
    client: GoogleAdsClient,
    customer_id: str,
    ad_group_resource_name: str,
) -> None:
    """Creates a new default shopping listing group for the specified ad group.

    A listing group is the Google Ads API representation of a "product group"
    described in the Google Ads user interface. The listing group will be added
    to the ad group using an "ad group criterion". The criterion will contain
    the bid for a given listing group.
    """
    ad_group_criterion_service: AdGroupCriterionServiceClient = (
        client.get_service("AdGroupCriterionService")
    )

    # Creates a new ad group criterion. This will contain the "default" listing
    # group (All products).
    ad_group_criterion_operation: AdGroupCriterionOperation = client.get_type(
        "AdGroupCriterionOperation"
    )
    ad_group_criterion: AdGroupCriterion = ad_group_criterion_operation.create
    ad_group_criterion.ad_group = ad_group_resource_name
    ad_group_criterion.status = client.enums.AdGroupCriterionStatusEnum.ENABLED
    ad_group_criterion.listing_group.type_ = (
        client.enums.ListingGroupTypeEnum.UNIT
    )
    #  Set the bid for products in this listing group unit.
    ad_group_criterion.cpc_bid_micros = 500000

    ad_group_criterion_response = (
        ad_group_criterion_service.mutate_ad_group_criteria(
            customer_id=customer_id, operations=[ad_group_criterion_operation]
        )
    )

    print(
        "Added an ad group criterion containing a listing group with "
        f"'{ad_group_criterion_response.results[0].resource_name}'."
    )


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description=(
            "Adds a standard shopping campaign, a shopping product ad "
            "group and a shopping product ad to the specified "
            "merchant account."
        )
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-m",
        "--merchant_center_account_id",
        type=int,
        required=True,
        help="The merchant center account ID.",
    )
    parser.add_argument(
        "-d",
        "--create_default_listing_group",
        action="store_true",
        default=False,
        help="Create a default listing group.",
    )
    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(
            googleads_client,
            args.customer_id,
            args.merchant_center_account_id,
            args.create_default_listing_group,
        )
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v21.enums.types import (
    call_tracking_display_location as gage_call_tracking_display_location,
)
from google.ads.googleads.v21.enums.types import call_type
from google.ads.googleads.v21.enums.types import google_voice_call_status


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.resources",
    marshal="google.ads.googleads.v21",
    manifest={
        "CallView",
    },
)


class CallView(proto.Message):
    r"""A call view that includes data for call tracking of call-only
    ads or call extensions.

    Attributes:
        resource_name (str):
            Output only. The resource name of the call view. Call view
            resource names have the form:

            ``customers/{customer_id}/callViews/{call_detail_id}``
        caller_country_code (str):
            Output only. Country code of the caller.
        caller_area_code (str):
            Output only. Area code of the caller. Null if
            the call duration is shorter than 15 seconds.
        call_duration_seconds (int):
            Output only. The advertiser-provided call
            duration in seconds.
        start_call_date_time (str):
            Output only. The advertiser-provided call
            start date time.
        end_call_date_time (str):
            Output only. The advertiser-provided call end
            date time.
        call_tracking_display_location (google.ads.googleads.v21.enums.types.CallTrackingDisplayLocationEnum.CallTrackingDisplayLocation):
            Output only. The call tracking display
            location.
        type_ (google.ads.googleads.v21.enums.types.CallTypeEnum.CallType):
            Output only. The type of the call.
        call_status (google.ads.googleads.v21.enums.types.GoogleVoiceCallStatusEnum.GoogleVoiceCallStatus):
            Output only. The status of the call.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    caller_country_code: str = proto.Field(
        proto.STRING,
        number=2,
    )
    caller_area_code: str = proto.Field(
        proto.STRING,
        number=3,
    )
    call_duration_seconds: int = proto.Field(
        proto.INT64,
        number=4,
    )
    start_call_date_time: str = proto.Field(
        proto.STRING,
        number=5,
    )
    end_call_date_time: str = proto.Field(
        proto.STRING,
        number=6,
    )
    call_tracking_display_location: (
        gage_call_tracking_display_location.CallTrackingDisplayLocationEnum.CallTrackingDisplayLocation
    ) = proto.Field(
        proto.ENUM,
        number=7,
        enum=gage_call_tracking_display_location.CallTrackingDisplayLocationEnum.CallTrackingDisplayLocation,
    )
    type_: call_type.CallTypeEnum.CallType = proto.Field(
        proto.ENUM,
        number=8,
        enum=call_type.CallTypeEnum.CallType,
    )
    call_status: (
        google_voice_call_status.GoogleVoiceCallStatusEnum.GoogleVoiceCallStatus
    ) = proto.Field(
        proto.ENUM,
        number=9,
        enum=google_voice_call_status.GoogleVoiceCallStatusEnum.GoogleVoiceCallStatus,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.enums",
    marshal="google.ads.googleads.v22",
    manifest={
        "ConversionAdjustmentTypeEnum",
    },
)


class ConversionAdjustmentTypeEnum(proto.Message):
    r"""Container for enum describing conversion adjustment types."""

    class ConversionAdjustmentType(proto.Enum):
        r"""The different actions advertisers can take to adjust the
        conversions that they already reported. Retractions negate a
        conversion. Restatements change the value of a conversion.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Represents value unknown in this version.
            RETRACTION (2):
                Negates a conversion so that its total value
                and count are both zero.
            RESTATEMENT (3):
                Changes the value of a conversion.
            ENHANCEMENT (4):
                Supplements an existing conversion with
                provided user identifiers and user agent, which
                can be used by Google to enhance the conversion
                count.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        RETRACTION = 2
        RESTATEMENT = 3
        ENHANCEMENT = 4


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.resources",
    marshal="google.ads.googleads.v21",
    manifest={
        "UserLocationView",
    },
)


class UserLocationView(proto.Message):
    r"""A user location view.

    User Location View includes all metrics aggregated at the
    country level, one row per country. It reports metrics at the
    actual physical location of the user by targeted or not targeted
    location. If other segment fields are used, you may get more
    than one row per country.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Output only. The resource name of the user location view.
            UserLocation view resource names have the form:

            ``customers/{customer_id}/userLocationViews/{country_criterion_id}~{targeting_location}``
        country_criterion_id (int):
            Output only. Criterion Id for the country.

            This field is a member of `oneof`_ ``_country_criterion_id``.
        targeting_location (bool):
            Output only. Indicates whether location was
            targeted or not.

            This field is a member of `oneof`_ ``_targeting_location``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    country_criterion_id: int = proto.Field(
        proto.INT64,
        number=4,
        optional=True,
    )
    targeting_location: bool = proto.Field(
        proto.BOOL,
        number=5,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2018 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This illustrates how to retrieve disapproved ads in a given campaign."""


import argparse
import sys

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.services.services.google_ads_service import (
    GoogleAdsServiceClient,
)
from google.ads.googleads.v22.services.types.google_ads_service import (
    SearchGoogleAdsRequest,
    SearchGoogleAdsResponse,
)
from google.ads.googleads.v22.resources.types.ad_group_ad import AdGroupAd
from google.ads.googleads.v22.resources.types.ad import Ad
from google.ads.googleads.v22.enums.types.policy_approval_status import (
    PolicyApprovalStatusEnum,
)


def main(client: GoogleAdsClient, customer_id: str, campaign_id: str) -> None:
    ga_service: GoogleAdsServiceClient = client.get_service("GoogleAdsService")

    query: str = f"""
        SELECT
          ad_group_ad.ad.id,
          ad_group_ad.ad.type,
          ad_group_ad.policy_summary.approval_status,
          ad_group_ad.policy_summary.policy_topic_entries
        FROM ad_group_ad
        WHERE
          campaign.id = {campaign_id}
          AND ad_group_ad.policy_summary.approval_status = DISAPPROVED"""

    request: SearchGoogleAdsRequest = client.get_type("SearchGoogleAdsRequest")
    request.customer_id = customer_id
    request.search_settings.return_total_results_count = True
    request.query = query

    results: SearchGoogleAdsResponse = ga_service.search(request=request)

    disapproved_enum: PolicyApprovalStatusEnum.PolicyApprovalStatus = (
        client.enums.PolicyApprovalStatusEnum.DISAPPROVED
    )

    print("Disapproved ads:")

    # Iterate over all ads in all rows returned and count disapproved ads.
    for result_row in results:
        # result_row: GoogleAdsRow = result_row <- Removed type hint
        ad_group_ad: AdGroupAd = result_row.ad_group_ad
        ad: Ad = ad_group_ad.ad
        policy_summary = ad_group_ad.policy_summary

        if not policy_summary.approval_status == disapproved_enum:
            continue

        print(
            f'Ad with ID "{ad.id}" and type "{ad.type_.name}" was '
            "disapproved with the following policy topic entries:"
        )

        # Display the policy topic entries related to the ad disapproval.
        for pol_entry in policy_summary.policy_topic_entries:
            # pol_entry: PolicyTopicEntry = pol_entry <- Removed type hint
            print(
                f'\ttopic: "{pol_entry.topic}", type "{pol_entry.type_.name}"'
            )

            # Display the attributes and values that triggered the policy
            # topic.
            for pol_evidence in pol_entry.evidences:
                # pol_evidence: PolicyTopicEvidence = pol_evidence <- Removed type hint
                for index, ev_text in enumerate(pol_evidence.text_list.texts):
                    # ev_text: str = ev_text <- Removed type hint
                    print(f"\t\tevidence text[{index}]: {ev_text}")

    print(f"\nNumber of disapproved ads found: {results.total_results_count}")


if __name__ == "__main__":
    parser: argparse.ArgumentParser = argparse.ArgumentParser(
        description=(
            "Lists disapproved ads for a given customer's specified "
            "campaign."
        )
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-i", "--campaign_id", type=str, required=True, help="The campaign ID."
    )
    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(
            googleads_client,
            args.customer_id,
            args.campaign_id,
        )
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v20.resources.types import customer_manager_link
from google.protobuf import field_mask_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.services",
    marshal="google.ads.googleads.v20",
    manifest={
        "MutateCustomerManagerLinkRequest",
        "MoveManagerLinkRequest",
        "CustomerManagerLinkOperation",
        "MutateCustomerManagerLinkResponse",
        "MoveManagerLinkResponse",
        "MutateCustomerManagerLinkResult",
    },
)


class MutateCustomerManagerLinkRequest(proto.Message):
    r"""Request message for
    [CustomerManagerLinkService.MutateCustomerManagerLink][google.ads.googleads.v20.services.CustomerManagerLinkService.MutateCustomerManagerLink].

    Attributes:
        customer_id (str):
            Required. The ID of the customer whose
            customer manager links are being modified.
        operations (MutableSequence[google.ads.googleads.v20.services.types.CustomerManagerLinkOperation]):
            Required. The list of operations to perform
            on individual customer manager links.
        validate_only (bool):
            If true, the request is validated but not
            executed. Only errors are returned, not results.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    operations: MutableSequence["CustomerManagerLinkOperation"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="CustomerManagerLinkOperation",
        )
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=3,
    )


class MoveManagerLinkRequest(proto.Message):
    r"""Request message for
    [CustomerManagerLinkService.MoveManagerLink][google.ads.googleads.v20.services.CustomerManagerLinkService.MoveManagerLink].

    Attributes:
        customer_id (str):
            Required. The ID of the client customer that
            is being moved.
        previous_customer_manager_link (str):
            Required. The resource name of the previous
            CustomerManagerLink. The resource name has the form:
            ``customers/{customer_id}/customerManagerLinks/{manager_customer_id}~{manager_link_id}``
        new_manager (str):
            Required. The resource name of the new manager customer that
            the client wants to move to. Customer resource names have
            the format: "customers/{customer_id}".
        validate_only (bool):
            If true, the request is validated but not
            executed. Only errors are returned, not results.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    previous_customer_manager_link: str = proto.Field(
        proto.STRING,
        number=2,
    )
    new_manager: str = proto.Field(
        proto.STRING,
        number=3,
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=4,
    )


class CustomerManagerLinkOperation(proto.Message):
    r"""Updates the status of a CustomerManagerLink.
    The following actions are possible:

    1. Update operation with status ACTIVE accepts a pending
        invitation.
    2. Update operation with status REFUSED declines a pending
        invitation.
    3. Update operation with status INACTIVE terminates link to
        manager.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            FieldMask that determines which resource
            fields are modified in an update.
        update (google.ads.googleads.v20.resources.types.CustomerManagerLink):
            Update operation: The link is expected to
            have a valid resource name.

            This field is a member of `oneof`_ ``operation``.
    """

    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=4,
        message=field_mask_pb2.FieldMask,
    )
    update: customer_manager_link.CustomerManagerLink = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="operation",
        message=customer_manager_link.CustomerManagerLink,
    )


class MutateCustomerManagerLinkResponse(proto.Message):
    r"""Response message for a CustomerManagerLink mutate.

    Attributes:
        results (MutableSequence[google.ads.googleads.v20.services.types.MutateCustomerManagerLinkResult]):
            A result that identifies the resource
            affected by the mutate request.
    """

    results: MutableSequence["MutateCustomerManagerLinkResult"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message="MutateCustomerManagerLinkResult",
        )
    )


class MoveManagerLinkResponse(proto.Message):
    r"""Response message for a CustomerManagerLink moveManagerLink.

    Attributes:
        resource_name (str):
            Returned for successful operations.
            Represents a CustomerManagerLink resource of the
            newly created link between client customer and
            new manager customer.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class MutateCustomerManagerLinkResult(proto.Message):
    r"""The result for the customer manager link mutate.

    Attributes:
        resource_name (str):
            Returned for successful operations.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Adds various types of negative criteria as exclusions at the customer level.

These criteria will be applied to all campaigns for the given customer.
"""


import argparse
import sys

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.resources.types.customer_negative_criterion import (
    CustomerNegativeCriterion,
)
from google.ads.googleads.v22.services.services.customer_negative_criterion_service import (
    CustomerNegativeCriterionServiceClient,
)
from google.ads.googleads.v22.services.types.customer_negative_criterion_service import (
    CustomerNegativeCriterionOperation,
    MutateCustomerNegativeCriteriaResponse,
)


def main(client: GoogleAdsClient, customer_id: str) -> None:
    """The main method that creates all necessary entities for the example.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
    """
    tragedy_criterion_op: CustomerNegativeCriterionOperation = client.get_type(
        "CustomerNegativeCriterionOperation"
    )
    tragedy_criterion: CustomerNegativeCriterion = tragedy_criterion_op.create
    # Creates a negative customer criterion excluding the content label type
    # of 'TRAGEDY'.
    tragedy_criterion.content_label.type_ = (
        client.enums.ContentLabelTypeEnum.TRAGEDY
    )

    placement_criterion_op: CustomerNegativeCriterionOperation = (
        client.get_type("CustomerNegativeCriterionOperation")
    )
    placement_criterion: CustomerNegativeCriterion = (
        placement_criterion_op.create
    )
    # Creates a negative customer criterion excluding the placement with URL
    # 'http://www.example.com'.
    placement_criterion.placement.url = "http://www.example.com"

    customer_negative_criterion_service: (
        CustomerNegativeCriterionServiceClient
    ) = client.get_service("CustomerNegativeCriterionService")

    # Issues a mutate request to add the negative customer criteria.
    response: MutateCustomerNegativeCriteriaResponse = (
        customer_negative_criterion_service.mutate_customer_negative_criteria(
            customer_id=customer_id,
            operations=[tragedy_criterion_op, placement_criterion_op],
        )
    )

    print(f"Added {len(response.results)} negative customer criteria:")
    for negative_criterion in response.results:
        print(f"Resource name: '{negative_criterion.resource_name}'")


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description=(
            "Adds various types of negative criteria as exclusions at the "
            "customer level. These criteria will be applied to all campaigns "
            "for the given customer."
        )
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(
            googleads_client,
            args.customer_id,
        )
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)
